from amuse.test import amusetest

from amuse.support.interface import InCodeComponentImplementation

from amuse.datamodel.indexing import *
from amuse.datamodel import indexing


class TestIndexing(amusetest.TestCase):
    def test1(self):
        self.assertEqual(2, number_of_dimensions_after_index(3, 1))
        self.assertEqual(3, number_of_dimensions_after_index(3, numpy.s_[0:3]))
        self.assertEqual(1, number_of_dimensions_after_index(3, combine_indices(3, 2)))
        self.assertEqual(0, number_of_dimensions_after_index(3, combine_indices(combine_indices(3, 2), 1)))
        self.assertEqual(3, indexing.number_of_dimensions_after_index(3, numpy.s_[1:2, ..., ...]))
        self.assertEqual(3, indexing.number_of_dimensions_after_index(3, numpy.s_[1:2, :, :]))

    def test2(self):
        a = numpy.arange(12).reshape(3, 4)
        self.assertEqual(a[combine_indices(0, 1)], a[0][1])
        self.assertEqual(a[combine_indices(1, 0)], a[1][0])
        self.assertTrue(numpy.all(a[combine_indices(1, numpy.s_[0:2])] == a[1][0:2]))
        indirect = combine_indices(0, 1)
        self.assertEqual(number_of_dimensions(a, indirect), 0)

    def test3(self):
        a = numpy.arange(12).reshape(3, 4)
        self.assertTrue(a[combine_indices(numpy.s_[0:2], 0)].shape, a[0:2][0].shape)
        self.assertTrue(numpy.all(a[combine_indices(numpy.s_[0:2], 0)] == a[0:2][0]))

    def test4(self):
        a = numpy.arange(12).reshape(3, 4)
        direct = a[1][:]
        indirect = a[combine_indices(1, indexing.normalize_slices(a[1].shape, numpy.s_[:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test5(self):
        a = numpy.arange(12).reshape(3, 4)
        direct = a[0:2][:]
        indirect = a[combine_indices(numpy.s_[0:2], indexing.normalize_slices(a[0:2].shape, numpy.s_[:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test6(self):
        a = numpy.arange(12).reshape(3, 4)
        direct = a[1:3][1:]
        indirect = a[combine_indices(numpy.s_[1:3], indexing.normalize_slices(a[1:3].shape, numpy.s_[1:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test7(self):
        a = numpy.arange(30).reshape(6, 5)
        direct = a[1:5:2][1:]
        indirect = a[combine_indices(numpy.s_[1:5:2], indexing.normalize_slices(a[1:5:2].shape, numpy.s_[1:]))]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test8(self):
        a = numpy.arange(30)
        direct = a[2:14:3][1:5:2]
        indirect = a[combine_indices(numpy.s_[2:14:3], numpy.s_[1:5:2])]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test9(self):
        a = numpy.arange(100)
        for s in range(0, 40):
            for e in range(40, 101):
                for step in range(1, 5):
                    direct = a[s:e:step][1:5:2]
                    indirect = a[combine_indices(numpy.s_[s:e:step],
                      indexing.normalize_slices(a[s:e:step].shape, numpy.s_[1:5:2]))]
                    self.assertEqual(indirect.shape, direct.shape)
                    self.assertTrue(numpy.all(indirect == direct))

    def test10(self):
        a = numpy.arange(60).reshape(5, 6, 2)
        direct = a[3][2][1]
        indirect = a[combine_indices(combine_indices(3, 2), 1)]
        self.assertEqual(indirect, direct)

    def test11(self):
        a = numpy.arange(60).reshape(5, 6, 2)
        direct = a[3]
        indirect = a[combine_indices(3, Ellipsis)]
        self.assertEqual(indirect.shape, direct.shape)
        self.assertTrue(numpy.all(indirect == direct))

    def test12(self):
        self.assertEqual((1, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:2, ..., ...]))
        self.assertEqual((1, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:2, :, :]))
        self.assertEqual((2, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:3, ..., ...]))
        self.assertEqual((2, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:3, :, :]))
        self.assertEqual((2, 1, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:3, 2:3, ...]))
        self.assertEqual((2, 1, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:3, 2:3, :]))

    def xtest13(self):
        combined_indices = combine_indices(numpy.s_[1:3], numpy.s_[:])
        self.assertEqual(combined_indices, numpy.s_[1:3:1])
        combined_indices = combine_indices(numpy.s_[:], numpy.s_[1:3])
        self.assertEqual(combined_indices, numpy.s_[1:3:1])
        combined_indices = combine_indices((numpy.s_[:], numpy.s_[:]), (numpy.s_[1:3], numpy.s_[1:2]))
        self.assertEqual(combined_indices, (numpy.s_[1:3:1], numpy.s_[1:2:1]))

        combined_indices = combine_indices((numpy.s_[0:2], numpy.s_[:]), (numpy.s_[1:3], numpy.s_[1:2]))
        self.assertEqual(combined_indices, (numpy.s_[1:2:1], numpy.s_[1:2:1]))

    def test14(self):
        self.assertEqual((5, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[:10, ..., ...]))
        self.assertEqual((5, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[..., :10, ...]))
        self.assertEqual((4, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[1:10, ..., ...]))
        self.assertEqual((1, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[-1:, ..., ...]))
        self.assertEqual((2, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[-2:, ..., ...]))
        self.assertEqual((1, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[-2:-1, ..., ...]))
        self.assertEqual((5, 4, 2), indexing.shape_after_index((5, 4, 2), numpy.s_[-10:, ..., ...]))

    def test15(self):
        a = numpy.arange(6).reshape(2, 3)
        indices = numpy.asarray([[True, False, True], [True, False, True]])
        direct = a[indices][list([1, 3])]
        combined = combine_indices(indices, [1, 3])
        indirect = a[combined]
        self.assertEqual(indirect, direct)

    def test16(self):
        self.assertEqual((4,), indexing.shape_after_index((2, 3), [[True, False, True], [True, False, True]]))
        self.assertEqual((1, 3), indexing.shape_after_index((2, 3), [True, False]))

    def test17(self):
        a = numpy.arange(6).reshape(2, 3)
        indices = numpy.asarray([True, False])
        direct = a[indices, 1:][0, 1:]
        combined = combine_indices(indexing.normalize_slices(a.shape, numpy.s_[indices, 1:]),
            indexing.normalize_slices(a[indices, 1:].shape, numpy.s_[0, 1:]))
        indirect = a[combined]
        self.assertEqual(indirect, direct)

    def test18(self):
        a = numpy.arange(6).reshape(2, 3)
        indices = numpy.asarray([True, False])
        direct = a[indices, 1:][0]
        combined = combine_indices(numpy.s_[indices, 1:], 0)
        indirect = a[combined]
        self.assertEqual(indirect, direct)

    def test19(self):
        a = numpy.arange(6).reshape(2, 3)
        indices = numpy.asarray([True, False])
        direct = a[:1, 1:][0]
        combined = combine_indices(numpy.s_[:1, 1:], 0)
        indirect = a[combined]
        self.assertEqual(indirect, direct)

    def test20(self):
        combined = combine_indices(slice(1, 199, None), slice(3, 5, None))
        self.assertEqual(combined, slice(4, 6, 1))

    def test21(self):
        shape = shape_after_index((200,), slice(4, 6, 1))
        self.assertEqual(shape, (2,))

    def xtest22(self):
        tiny = list(range(2))
        small = list(range(10))
        big = list(range(1000))

        # combining slicings w negative stops not possible! e.g. ((7,-1),(2,3),(9,10,1))
        # (without normalize)
        slicings = [((9, 19), (5, 9, 2), (14, 18, 2)),
                   ((7, 19, 2), (5, 9, 1), (17, 19, 2)),
                   ((1, None), (1, 10), (2, 11, 1)),
                   ((7, None), (1, 10), (8, 17, 1)),
                   ((None, 12), (3, 5), (3, 5, 1)),
                   ((None, 12), (3, 15), (3, 12, 1)),
                   ((None, None), (3, 5), (3, 5, 1)),
                   ((None, None), (3, 15), (3, 15, 1)),
                   ((None, None), (None, 15), (0, 15, 1)),
                   ((None, None), (None, None), (0, None, 1)),
                   ((9, None), (None, None), (9, None, 1)),
                   ((9, None), (6, None), (15, None, 1)),
                   ((9, None), (None, 40), (9, 49, 1)),
                   ((1, None), (None, 40), (1, 41, 1)),
                   ((9, 16), (None, 40), (9, 16, 1)),
                   ((1, 16), (None, 40), (1, 16, 1)),
                   ((49, 16), (None, 40), (16, 16, 1)),
                   ((41, 66), (None, 40), (41, 66, 1)),
                 ]

        for t1, t2, t3 in slicings:
            s1 = slice(*t1)
            s2 = slice(*t2)
            s3 = slice(*t3)
            self.assertEqual(combine_slices(s1, s2), t3)
            self.assertTrue(tiny[s1][s2] == tiny[s3])
            self.assertTrue(small[s1][s2] == small[s3])
            self.assertTrue(big[s1][s2] == big[s3])

    def xtest23(self):
        import random
        random.seed(123456)
        tiny = list(range(2))
        small = list(range(20))
        big = list(range(2000))

        Ntest = 1000
        start0 = [random.randint(0, 20) for x in range(Ntest)]
        stop0 = [random.randint(15, 50) for x in range(Ntest)]
        step0 = [random.randint(1, 3) for x in range(Ntest)]
        start1 = [random.randint(0, 10) for x in range(Ntest)]
        stop1 = [random.randint(5, 25) for x in range(Ntest)]
        step1 = [random.randint(1, 3) for x in range(Ntest)]

        slicings = []
        for x in zip(start0, stop0, step0, start1, stop1, step1):
            slicings.append(((x[0], x[1], x[2]), (x[3], x[4], x[5])))

        for t1, t2 in slicings:
            s1 = slice(*t1)
            s2 = slice(*t2)
            t3 = combine_slices(s1, s2)
            s3 = slice(*t3)
            self.assertTrue(tiny[s1][s2] == tiny[s3])
            self.assertTrue(small[s1][s2] == small[s3])
            self.assertTrue(big[s1][s2] == big[s3])

    def test24(self):
        import random
        random.seed(123456)
        tiny = list(range(2))
        small = list(range(20))
        big = list(range(2000))

        Ntest = 1000
        stop0 = [random.randint(0, 20) for x in range(Ntest)]
        start0 = [random.randint(15, 50) for x in range(Ntest)]
        step0 = [random.randint(-3, -1) for x in range(Ntest)]
        start1 = [random.randint(0, 10) for x in range(Ntest)]
        stop1 = [random.randint(5, 25) for x in range(Ntest)]
        step1 = [random.randint(1, 3) for x in range(Ntest)]

        slicings = []
        for x in zip(start0, stop0, step0, start1, stop1, step1):
            slicings.append(((x[0], x[1], x[2]), (x[3], x[4], x[5])))

        for t1, t2 in slicings:
            s1 = slice(*t1)
            s2 = slice(*t2)

            t3 = combine_slices(normalize_slices(len(tiny), s1), normalize_slices(len(tiny[s1]), s2))
            s3 = slice(*t3)
            self.assertTrue(tiny[s1][s2] == tiny[s3])

            t3 = combine_slices(normalize_slices(len(small), s1), normalize_slices(len(small[s1]), s2))
            s3 = slice(*t3)
            self.assertTrue(small[s1][s2] == small[s3])
            t3 = combine_slices(normalize_slices(len(big), s1), normalize_slices(len(big[s1]), s2))
            s3 = slice(*t3)
            self.assertTrue(big[s1][s2] == big[s3])

    def test25(self):
        import random
        random.seed(123456)
        tiny = list(range(2))
        small = list(range(20))
        big = list(range(2000))

        Ntest = 1000
        stop0 = [random.randint(0, 20) for x in range(Ntest)]
        start0 = [random.randint(15, 50) for x in range(Ntest)]
        step0 = [random.randint(-3, -1) for x in range(Ntest)]
        stop1 = [random.randint(0, 10) for x in range(Ntest)]
        start1 = [random.randint(5, 25) for x in range(Ntest)]
        step1 = [random.randint(-3, -1) for x in range(Ntest)]

        slicings = []
        for x in zip(start0, stop0, step0, start1, stop1, step1):
            slicings.append(((x[0], x[1], x[2]), (x[3], x[4], x[5])))

        for t1, t2 in slicings:
            s1 = slice(*t1)
            s2 = slice(*t2)

            t3 = combine_slices(normalize_slices(len(tiny), s1), normalize_slices(len(tiny[s1]), s2))
            s3 = slice(*t3)
            # ~ print(s1,s2,s3)
            self.assertTrue(tiny[s1][s2] == tiny[s3])

            t3 = combine_slices(normalize_slices(len(small), s1), normalize_slices(len(small[s1]), s2))
            s3 = slice(*t3)
            self.assertTrue(small[s1][s2] == small[s3])
            t3 = combine_slices(normalize_slices(len(big), s1), normalize_slices(len(big[s1]), s2))
            s3 = slice(*t3)
            self.assertTrue(big[s1][s2] == big[s3])

    def test26(self):
        oned = numpy.zeros(5)
        threed = numpy.zeros((4, 5, 6))
        for index in [0, [1], [1, 2], [[1, 2], [2, 3]], [[2]], [[0, 1]]]:
            i = numpy.array(index)
            self.assertEqual(len(oned[i].shape), number_of_dimensions_after_index(1, i))
        for index in [0, [1], [1, 2], [[1, 2], [2, 3]], [[2]], [[2, 1]]]:
            i = numpy.array(index)
            self.assertEqual(len(threed[i].shape), number_of_dimensions_after_index(3, i))

    def test27(self):
        oned = numpy.zeros(5)
        threed = numpy.zeros((4, 5, 6))
        for index in [0, [1], [1, 2], [[1, 2], [2, 3]], [[2]], [[0, 1]]]:
            i = numpy.array(index)
            self.assertEqual(oned[i].shape, shape_after_index(oned.shape, i))
        for index in [0, [1], [1, 2], [[1, 2], [2, 3]], [[2]], [[2, 1]], [[[[0], [1], [1]]]]]:
            i = numpy.array(index)
            self.assertEqual(threed[i].shape, shape_after_index(threed.shape, i))

    def test28(self):
        twod = numpy.zeros((5, 6))
        threed = numpy.zeros((4, 5, 6))
        for _i, _j in [([0], [1]), ([0, 2], [1, 3]), ([0, 2], [1, 3]), ([[0, 1], [1, 2]], [[2, 3], [3, 4]])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(len(twod[i, j].shape), number_of_dimensions_after_index(2, (i, j)))
        for _i, _j in [([0], [1]), ([0, 2], [1, 3]), ([0, 2], [1, 3]), ([[0, 1], [1, 2]], [[2, 3], [3, 4]])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(len(threed[i, j].shape), number_of_dimensions_after_index(3, (i, j)))

    def test29(self):
        twod = numpy.zeros((5, 6))
        for _i, _j in [([0], [1]), ([0, 2], [1, 3]), ([0, 2], [1, 3]), ([[0, 1], [1, 2]], [[2, 3], [3, 4]])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(twod[i, j].shape, shape_after_index(twod.shape, (i, j)))

        threed = numpy.zeros((4, 5, 6))
        for _i, _j in [([0], [1]), ([0, 2], [1, 3]), ([0, 2], [1, 3]), ([[0, 1], [1, 2]], [[2, 3], [3, 4]])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(threed[i, j].shape, shape_after_index(threed.shape, (i, j)))

        fourd = numpy.zeros((4, 5, 6, 7))
        for _i, _j in [([0], [1]), ([0, 2], [1, 3])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(fourd[i, j].shape, shape_after_index(fourd.shape, (i, j)))
        for _i, _j in [([0, 2], [1, 3])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(fourd[i, Ellipsis, j].shape, shape_after_index(fourd.shape, (i, Ellipsis, j)))
        for _i, _j in [([0, 2], [1, 3])]:
            i = numpy.array(_i)
            j = numpy.array(_j)
            self.assertEqual(fourd[i, slice(None), j].shape, shape_after_index(fourd.shape, (i, slice(None), j)))


class TestSplitOverDimensions(amusetest.TestCase):

    def test1(self):
        dimension_values = [
            [3, 4, 5, 6],
            ['a', 'b', 'c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(0, dimension_values)
        self.assertEqual(len(split_dimension_values), 2)
        self.assertEqual(split_dimension_values[0], 3)
        self.assertEqual(split_dimension_values[1], ['a', 'b', 'c'])

    def test2(self):
        dimension_values = [
            [3, 4, 5, 6],
            ['a', 'b', 'c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions((1, 2), dimension_values)
        self.assertEqual(len(split_dimension_values), 2)
        self.assertEqual(split_dimension_values[0], 4)
        self.assertEqual(split_dimension_values[1], 'c')

    def test3(self):
        dimension_values = [
            [3, 4, 5, 6],
            ['a', 'b', 'c']
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(0, 2), dimension_values)
        self.assertEqual(len(split_dimension_values), 2)
        self.assertEqual(split_dimension_values[0], [3, 4])
        self.assertEqual(split_dimension_values[1], ['a', 'b', 'c'])

    def test4(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8, ]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(1, 7, 2), dimension_values)
        self.assertEqual(len(split_dimension_values), 1)
        self.assertEqual(split_dimension_values[0], [1, 3, 5])

    def test5(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(-2, 10), dimension_values)
        self.assertEqual(split_dimension_values[0], [8, 9])
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(-3, 3, -1), dimension_values)
        self.assertEqual(split_dimension_values[0], [7, 6, 5, 4])

    def test6(self):
        dimension_values = [
            [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(5, None), dimension_values)
        self.assertEqual(split_dimension_values[0], [5, 6, 7, 8, 9])

    def test7(self):
        dimension_values = [
            [0, 1],
            [0, 1, 2],
            [0]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions(slice(1, 2), dimension_values)
        self.assertEqual(split_dimension_values[0], [1])
        self.assertEqual(split_dimension_values[1], [0, 1, 2])
        self.assertEqual(split_dimension_values[2], [0])

    def test8(self):
        dimension_values = [
            [0, 1],
            [0, 1, 2],
            [0]
        ]
        split_dimension_values = indexing.split_numpy_index_over_dimensions((Ellipsis, 0), dimension_values)
        self.assertEqual(split_dimension_values[0], [0, 1])
        self.assertEqual(split_dimension_values[1], [0, 1, 2])
        self.assertEqual(split_dimension_values[2], 0)

        split_dimension_values = indexing.split_numpy_index_over_dimensions((slice(None), slice(None), 0), dimension_values)
        self.assertEqual(split_dimension_values[0], [0, 1])
        self.assertEqual(split_dimension_values[1], [0, 1, 2])
        self.assertEqual(split_dimension_values[2], 0)

        split_dimension_values = indexing.split_numpy_index_over_dimensions((Ellipsis, 0, Ellipsis), dimension_values)
        self.assertEqual(split_dimension_values[0], [0, 1])
        self.assertEqual(split_dimension_values[1], 0)
        self.assertEqual(split_dimension_values[2], [0])



from amuse.test import amusetest

import os.path
from amuse import io
from amuse.units import units


class TestsForTicket118(amusetest.TestCase):

    def test1(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')
        set = io.read_set_from_file(filename, 'dyn')

        self.assertEqual(len(set), 10)

    def test2(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')

        root = io.read_set_from_file(filename, 'dyn', return_children=False)

        self.assertFalse(root is None)

    def test3(self):
        filename = os.path.join(os.path.dirname(__file__), 'FinalSnapshot.out')
        set = io.read_set_from_file(filename, 'dyn')
        print(set)
        print("set[0].parent.mass", set[0].parent.mass)
        self.assertAlmostRelativeEquals(0.000227826766314251919 * 617.75586357299929284496, set[9].mass.value_in(units.MSun), 12)
        self.assertAlmostRelativeEquals(617.75586357299929284496 * 0.953575109205781479,  set[0].parent.mass.value_in(units.MSun), 12)
        # self.assertTrue(False)



import numpy

from amuse.test import amusetest
from amuse.units import units, nbody_system
from amuse.ic.brokenimf import *

# Instead of random, use evenly distributed numbers, just for testing
default_options = dict(random=False)


class TestMultiplePartIMF(amusetest.TestCase):

    def test1(self):
        print("Test MultiplePartIMF with default mass_boundaries and alphas, i.e. Salpeter")
        instance = MultiplePartIMF(mass_max=100.0 | units.MSun)
        self.assertEqual(instance.mass_boundaries, [0.1, 100.0] | units.MSun)
        self.assertEqual(instance.alphas, [-2.35])
        self.assertEqual(instance.number_of_bins, 1)
        self.assertEqual(instance.fraction_per_bin, [1.0])
        self.assertEqual(instance.cumulative_fractions, [0.0, 1.0])

        self.assertAlmostEqual(instance.mass([0.0]), 0.1 | units.MSun)
        self.assertAlmostEqual(instance.mass([1.0]), 100.0 | units.MSun)
        self.assertAlmostEqual(instance.mass_mean(), 0.351 | units.MSun, 3)

    def test2(self):
        print("Test MultiplePartIMF with mass_boundaries and alphas")
        instance = MultiplePartIMF(mass_boundaries=[1.0, 10.0, 100.0] | units.MSun,
            alphas=[1.3, -3.3], **default_options)
        self.assertEqual(instance.mass_boundaries, [1.0, 10.0, 100.0] | units.MSun)
        self.assertEqual(instance.alphas, [1.3, -3.3])
        self.assertEqual(instance.number_of_bins, 2)
        self.assertAlmostEqual(instance.fraction_per_bin, numpy.array([0.5, 0.5]))
        self.assertEqual(instance.cumulative_fractions, [0.0, 0.5, 1.0])

        self.assertAlmostEqual(instance.mass([0.0]), 1.0 | units.MSun)
        self.assertAlmostEqual(instance.mass([0.5]), 10.0 | units.MSun)
        self.assertAlmostEqual(instance.mass([1.0]), 100.0 | units.MSun)

        self.assertAlmostEqual(instance.mass_mean(), 11.9457684987 | units.MSun)
        self.assertAlmostEqual(instance.mass_mean(), instance.next_mass(10000).mean(), 2)

    def test3(self):
        print("Test new_broken_power_law_mass_distribution with default mass_boundaries and alphas, i.e. Salpeter")
        masses = new_broken_power_law_mass_distribution(10000, mass_max=100.0 | units.MSun, **default_options)
        self.assertTrue((masses >= 0.1 | units.MSun).all())
        self.assertTrue((masses <= 100.0 | units.MSun).all())
        self.assertAlmostEqual(min(masses), 0.1 | units.MSun)
        self.assertAlmostEqual(max(masses), 100.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[0.1, 100.0] | units.MSun,
            alphas=[-2.35]).mass_mean()
        self.assertAlmostEqual(mass_mean, 0.35136877959 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 0.351 | units.MSun, 1)

    def test4(self):
        print("Test new_broken_power_law_mass_distribution with mass_boundaries and alphas")
        masses = new_broken_power_law_mass_distribution(10000,
            mass_boundaries=[1.0, 10.0, 100.0] | units.MSun,
            alphas=[1.3, -3.3], **default_options)
        self.assertTrue((masses >= 1.0 | units.MSun).all())
        self.assertTrue((masses <= 100.0 | units.MSun).all())
        self.assertAlmostEqual(min(masses), 1.0 | units.MSun)
        self.assertAlmostEqual(max(masses), 100.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[1.0, 10.0, 100.0] | units.MSun,
            alphas=[1.3, -3.3]).mass_mean()
        self.assertAlmostEqual(mass_mean, 11.9457684987 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 11.9457684987 | units.MSun, 1)

    def test5(self):
        print("Test new_scalo_mass_distribution")
        masses = new_scalo_mass_distribution(10000, **default_options)
        self.assertTrue((masses >= 0.1 | units.MSun).all())
        self.assertTrue((masses <= 125.0 | units.MSun).all())
        self.assertAlmostEqual(min(masses), 0.1 | units.MSun)
        self.assertAlmostEqual(max(masses), 125.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[0.10, 0.18, 0.42, 0.62, 1.18, 3.5, 125.0] | units.MSun,
            alphas=[1.6, -1.01, -2.75, -2.08, -3.5, -2.63]).mass_mean()
        self.assertAlmostEqual(mass_mean, 0.487756751788 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 0.487756751788 | units.MSun, 1)

    def test6(self):
        print("Test new_miller_scalo_mass_distribution")
        masses = new_miller_scalo_mass_distribution(10000, **default_options)
        self.assertTrue((masses >= 0.1 | units.MSun).all())
        self.assertTrue((masses <= 125.0 | units.MSun).all())
        self.assertAlmostEqual(min(masses), 0.1 | units.MSun)
        self.assertAlmostEqual(max(masses), 125.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[0.1, 1.0, 2.0, 10.0, 125.0] | units.MSun,
            alphas=[-1.25, -2.0, -2.3, -3.3]).mass_mean()
        self.assertAlmostEqual(mass_mean, 0.885783055149 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 0.885783055149 | units.MSun, 1)

    def test7(self):
        print("Test new_kroupa_mass_distribution")
        masses = new_kroupa_mass_distribution(10000, **default_options)
        self.assertTrue((masses >= 0.01 | units.MSun).all())
        roundoff = 1.0 + 1.0e-12
        self.assertTrue((masses <= (100.0 * roundoff) | units.MSun).all())
        self.assertAlmostEqual(min(masses), 0.01 | units.MSun)
        self.assertAlmostEqual(max(masses), 100.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[0.01, 0.08, 0.5, 100.0] | units.MSun,
            alphas=[-0.3, -1.3, -2.3]).mass_mean()
        self.assertAlmostEqual(mass_mean, 0.376175542639 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 0.376175542639 | units.MSun, 1)

    def test8(self):
        print("Test with problematic alphas (new_salpeter_mass_distribution would give zero division errors)")
        masses = new_broken_power_law_mass_distribution(10000,
            mass_boundaries=[1.0, 10.0, 100.0] | units.MSun,
            alphas=[-1, -2], **default_options)
        self.assertTrue((masses >= 1.0 | units.MSun).all())
        roundoff = 1.0 + 1.0e-12
        self.assertTrue((masses <= (100.0 * roundoff) | units.MSun).all())
        self.assertAlmostEqual(min(masses), 1.0 | units.MSun)
        self.assertAlmostEqual(max(masses), 100.0 | units.MSun)

        mass_mean = MultiplePartIMF(mass_boundaries=[1.0, 10.0, 100.0] | units.MSun,
            alphas=[-1, -2]).mass_mean()
        self.assertAlmostEqual(mass_mean, 10.0 | units.MSun)
        self.assertAlmostRelativeEqual(masses.mean(), 10.0 | units.MSun, 1)

        masses = new_broken_power_law_mass_distribution(101,
            mass_boundaries=[1.0, 100.0] | units.MSun,
            alphas=[-1], **default_options)
        self.assertAlmostEqual(masses.median(), 10.0 | units.MSun)



import numpy

from amuse.test import amusetest
from amuse.units import units, nbody_system, constants

from amuse.ext.rotating_bridge import Rotating_Bridge, inertial_to_rotating, rotating_to_inertial

from amuse.ext.composition_methods import *

from amuse.datamodel import Particles


class drift_without_gravity(object):
    """
    This class evolves the motion of the test particles (no gravity) 
    """

    def __init__(self, particles, time=0 | units.Myr):
        self.particles = particles
        self.model_time = time

    def evolve_model(self, t_end):
        dt = t_end - self.model_time
        self.particles.position += self.particles.velocity*dt
        self.model_time = t_end

    @property
    def potential_energy(self):
        return quantities.zero

    @property
    def kinetic_energy(self):
        return (0.5*self.particles.mass*self.particles.velocity.lengths()**2).sum()


class TestRotatingBridge(amusetest.TestCase):
    def test1(self):
        p0 = Particles(1)
        p0.position = [0.0, 1.0, 0.0]
        p0.velocity = [1.0, 0.0, 0.0]

        pos1 = p0.position.copy()
        vel1 = p0.velocity.copy()

        omega = 1.
        N = 0.25
        dt = 0.01
        method = SPLIT_6TH_SS_M13

        tend = N*2*numpy.pi/omega
        dt = dt*2*numpy.pi/omega

        pr = inertial_to_rotating(0, omega, p0)

        drift = drift_without_gravity(pr, time=0)
        sys = Rotating_Bridge(omega, timestep=dt, method=method)
        sys.add_system(drift)

        sys.evolve_model(tend)

        pi = rotating_to_inertial(tend, omega, pr)

        pos2 = pi.position.copy()

        self.assertAlmostEqual(sys.model_time, tend)
        self.assertAlmostEqual(pos1+vel1*tend, pos2, 12)

        pi.velocity = -pi.velocity

        pr = inertial_to_rotating(0, -omega, pi)
        drift = drift_without_gravity(pr, time=0)
        sys = Rotating_Bridge(-omega, timestep=dt, method=method)
        sys.add_system(drift)

        sys.evolve_model(tend)

        pi = rotating_to_inertial(tend, -omega, pr)

        pos3 = pi.position.copy()

        self.assertAlmostEqual(pos1, pos3, 12)

    def test2(self):
        p0 = Particles(1)
        p0.position = [0.0, 1.0, 0.0] | units.m
        p0.velocity = [1.0, 0.0, 0.0] | units.m/units.s

        pos1 = p0.position.copy()
        vel1 = p0.velocity.copy()

        omega = 1. | units.s**-1
        N = 0.25
        dt = 0.01
        method = SPLIT_6TH_SS_M13

        tend = N*2*numpy.pi/omega
        dt = dt*2*numpy.pi/omega

        pr = inertial_to_rotating(0*dt, omega, p0)

        drift = drift_without_gravity(pr, time=0 | units.s)
        sys = Rotating_Bridge(omega, timestep=dt, method=method)
        sys.add_system(drift)

        sys.evolve_model(tend)

        pi = rotating_to_inertial(tend, omega, pr)

        pos2 = pi.position.copy()

        self.assertAlmostEqual(sys.model_time, tend)
        self.assertAlmostEqual(pos1+vel1*tend, pos2, 12)

        pi.velocity = -pi.velocity

        pr = inertial_to_rotating(0*dt, -omega, pi)
        drift = drift_without_gravity(pr, time=0 | units.s)
        sys = Rotating_Bridge(-omega, timestep=dt, method=method)
        sys.add_system(drift)

        sys.evolve_model(tend)

        pi = rotating_to_inertial(tend, -omega, pr)

        pos3 = pi.position.copy()

        self.assertAlmostEqual(pos1, pos3, 12)



from amuse.test import amusetest
from amuse.units import units
from amuse.units import quantities
from amuse.support import interface
from amuse.support import exceptions

from amuse import datamodel


class ExampleParticlesInterface(interface.InCodeComponentImplementation):
    """This is an example class to demonstrate how to work with incode particle sets
    using the object mapping interface.

    The particle set mapping has two functions:

    1) Convert function calls to attribute access. Setting or getting
       an attribute is converted to a call to the code.

          particles[0].mass = 10 | units.kg
          -->
          code.set_mass(index_of_the_particle = 0, mass = 10)

    2) Convert ids from the code into particle keys. All particles
       in AMUSE have an unique key, this key is mapped to the corresponding
       id of that particle in a code by the particle set. As every particle
       set mapping is unique for a code, every code has a mapping table (and
       different codes can have differen ids for the same particle, the key
       to id mapping will ensure that attribute values are assigned correctly)

    This example class is not connected to a real code, instead it 
    provides all necessary methods to create, delete and update particles. 

    For simplicity all attributes are stored in quantities (values with units),
    no unit conversion is provided.

    In this example code particles have an id (id),  position (x,y and z) and a
    mass attribute (mass).

    See the ExampleParticlesInterfaceTests class for examples / tests of use.
    """

    def log(self, message, *arguments):
        print("IN CODE >>", message.format(*arguments))

    def __init__(self):
        super(type(self), self).__init__(None)
        self.mapping_from_id_to_particle = {}
        self.highest_id = 0
        self.log("initialized the code")

    def define_particle_sets(self, builder):
        """In this method we define the particle set and all attribute.
        We specify wich methods to call for creating and deleting particles.
        We specify the methods to access attributes of the particles
        We specify the methods to create subsets or links

        Redefined from CodeInterface.
        """

        # first we define the name of the set and wich parameter will be used
        # to specify the id of a particle in the code
        builder.define_set('particles', 'index_of_the_particle')

        # we define new and delete methods
        builder.set_new('particles', 'new_particle')
        builder.set_delete('particles', 'delete_particle')

        # we define the methods to get and set
        # attributes, note these attributes may overlap
        # we need to specify the returned attribute names for the
        # get functions as the framework cannot derive these from
        # python methods (python methods do not have named return values
        builder.add_setter('particles', 'set_state')
        builder.add_getter('particles', 'get_state', names=('mass', 'x', 'y', 'z'))
        builder.add_setter('particles', 'set_mass')
        builder.add_getter('particles', 'get_mass', names=('mass',))
        builder.add_setter('particles', 'set_position')
        builder.add_getter('particles', 'get_position', names=('x', 'y', 'z'))

        builder.add_method('particles',  'get_list', 'element_list')

        self.log("defined the particle set with name {0!r}", 'particles')

    def new_particle(self, mass, x, y, z):
        """Creates new particles.

        Note: the parameternames are directly coupled to the attribute names
        in the particle set

        Note: the arguments are all arrays
        """

        result = []
        for mass_element, x_element, y_element, z_element in zip(mass, x, y, z):
            particle = [self.highest_id, mass_element, x_element, y_element, z_element]
            self.mapping_from_id_to_particle[self.highest_id] = particle
            result.append(self.highest_id)

            self.log("created new particle with id {0}, (mass={1}, x={2}, y={3}, z={4})", self.highest_id, mass_element, x_element, y_element, z_element)

            self.highest_id += 1

        return result

    def delete_particle(self, index_of_the_particle):
        """Delete particles in array index_of_the_particle

        Note: the parametername index_of_the_particle was specified in the "define_particle_sets" method
        """

        for x in index_of_the_particle:
            del self.mapping_from_id_to_particle[x]

            self.log("deleted the particle with id {0}", x)

    def set_state(self, index_of_the_particle, mass, x, y, z):
        """Sets the mass and the position of a particle.

        Note: the arguments are arrays
        """

        for index_element, mass_element, x_element, y_element, z_element in zip(index_of_the_particle, mass, x, y, z):
            particle = self.mapping_from_id_to_particle[index_element]
            particle[1] = mass
            particle[2] = x
            particle[3] = y
            particle[4] = z

            self.log("updated state of particle with id {0}", index_element)

    def get_state(self, index_of_the_particle):
        """Returns arrays for the mass, x, y and z values
        """

        massresult = quantities.AdaptingVectorQuantity()
        xresult = quantities.AdaptingVectorQuantity()
        yresult = quantities.AdaptingVectorQuantity()
        zresult = quantities.AdaptingVectorQuantity()

        for index_element in index_of_the_particle:
            particle = self.mapping_from_id_to_particle[index_element]

            massresult.append(particle[1])
            xresult.append(particle[2])
            yresult.append(particle[3])
            zresult.append(particle[4])

            self.log("retrieved state of particle with id {0}", index_element)

        return massresult, xresult, yresult, zresult

    def get_mass(self, index_of_the_particle):
        """Returns an array for the masses of the indices int the index_of_the_particle array
        """

        massresult = quantities.AdaptingVectorQuantity()

        for index_element in index_of_the_particle:
            particle = self.mapping_from_id_to_particle[index_element]
            massresult.append(particle[1])

            self.log("retrieved mass of particle with id {0} (mass = {1})", index_element, particle[1])

        return massresult

    def set_mass(self, index_of_the_particle, mass):
        """Sets the mass and the position of a particle.

        Note: the arguments are arrays
        """

        for index_element, mass_element in zip(index_of_the_particle, mass):
            particle = self.mapping_from_id_to_particle[index_element]

            particle[1] = mass

            self.log("updated mass of particle with id {0} (mass = {1})", index_element, particle[1])

    def get_position(self, index_of_the_particle):
        """Returns an array of the positions for the indices in index_of_the_particle
        """

        xresult = quantities.AdaptingVectorQuantity()
        yresult = quantities.AdaptingVectorQuantity()
        zresult = quantities.AdaptingVectorQuantity()

        for index_element in index_of_the_particle:
            particle = self.mapping_from_id_to_particle[index_element]

            xresult.append(particle[2])
            yresult.append(particle[3])
            zresult.append(particle[4])

            self.log("retrieved position of particle with id {0} (x = {1}, y = {2}, z = {3})", index_element, particle[2], particle[3], particle[4])
        return xresult, yresult, zresult

    def set_position(self, index_of_the_particle, x, y, z):
        """Sets the mass and the position of a particle.

        Note: the arguments are arrays
        """

        for index_element, x_element, y_element, z_element in zip(index_of_the_particle, x, y, z):
            particle = self.mapping_from_id_to_particle[index_element]

            particle[2] = x
            particle[3] = y
            particle[4] = z

            self.log("updated position of particle with id {0} (x = {1}, y = {2}, z = {3})", index_element, particle[2], particle[3], particle[4])

    def get_list_size(self, index_of_the_particle):
        """Returns the inclusive range of indices in the
        list of element coupled to a particle.
        """
        return (0, 9)

    def get_list_element(self, index_in_the_list, index_of_the_particle):
        """Returns an array of the positions for the indices in index_of_the_particle
        """
        if not hasattr(index_in_the_list, '__iter__'):
            index_in_the_list = [index_in_the_list,]
        if not hasattr(index_of_the_particle, '__iter__'):
            index_of_the_particle = [index_of_the_particle,]
        value1 = quantities.AdaptingVectorQuantity()
        value2 = quantities.AdaptingVectorQuantity()

        for index_of_one_particle, index_of_one_element in zip(index_of_the_particle, index_in_the_list):

            value1.append(index_of_one_particle | units.none)
            value2.append(index_of_one_element | units.none)

        return value1, value2

    def get_list(self, index_of_the_particle):
        if hasattr(index_of_the_particle, '__iter__'):
            return [self._create_new_grid(self.specify_list, index_of_the_particle=x) for x in index_of_the_particle]
        else:
            return self._create_new_grid(self.specify_list, index_of_the_particle=index_of_the_particle)

    def specify_list(self, definition, index_of_the_particle=0):
        definition.set_grid_range('get_list_size')
        definition.add_getter('get_list_element', names=('value1', 'value2'))
        definition.define_extra_keywords({'index_of_the_particle': index_of_the_particle})


class ExampleParticlesInterfaceTests(amusetest.TestCase):
    """This class runs tests on the example particles interface
    class.

    """

    def test1(self):
        """
        In this test we will add and remove a particle from
        the particle set of the code.

        Adding a particle to the set will result in creating
        a new particle in the code.

        Removing a particle form the set will result in deleting
        the corresponding particle in the code.
        """

        self.log("adding and removing of a particle")

        instance = ExampleParticlesInterface()
        self.assertEqual(len(instance.particles), 0)

        # we create a particle in our script
        # all attributes of this particle are stored in the python space
        # when creating a particle you can set it's key
        # or let the system determine a unique key
        # to set the key in the script do: core.Particle(1000) where 1000 is the key

        theParticle = datamodel.Particle()
        theParticle.mass = 10 | units.kg
        theParticle.x = 0.1 | units.m
        theParticle.y = 0.2 | units.m
        theParticle.z = 0.5 | units.m

        self.log("Adding particle with key {0}", theParticle.key)
        instance.particles.add_particle(theParticle)

        print(instance.particles.index_in_code)
        self.assertEqual(len(instance.particles), 1)

        self.log("Removing particle with key {0}", theParticle.key)
        instance.particles.remove_particle(theParticle)

        self.assertEqual(len(instance.particles), 0)

    def test2(self):
        """
        In this test we will set and get different properties
        of the particle.

        To limit overhead, the system will use the set_* or get_* calls 
        that are the closests match to the attributes queries.
        """
        self.log("accessing attributes of a particle")
        instance = ExampleParticlesInterface()
        self.assertEqual(len(instance.particles), 0)

        theParticle = datamodel.Particle()
        theParticle.mass = 10 | units.kg
        theParticle.x = 0.1 | units.m
        theParticle.y = 0.2 | units.m
        theParticle.z = 0.5 | units.m

        instance.particles.add_particle(theParticle)

        self.log("Getting the mass of particle with key {0}, get_mass should be called", theParticle.key)
        self.assertEqual(instance.particles[0].mass, 10 | units.kg)

        self.log("Getting the position of particle with key {0}, get_position should be called", theParticle.key)
        self.assertEqual(instance.particles[0].position, [0.1, 0.2, 0.5] | units.m)

        self.log("Getting the only the x attribute of particle with key {0}, get_position should be called (y and z are discarded)", theParticle.key)
        self.assertEqual(instance.particles[0].x, 0.1 | units.m)

        self.log("Setting the position of particle with key {0}, set_position should be called", theParticle.key)
        instance.particles[0].position = [0.2, 0.3, 0.6] | units.m

        self.log("Setting the x of particle with key {0}, should fail as no function can set x and no others", theParticle.key)

        def block():
            instance.particles[0].x = 0.1 | units.m
        self.assertRaises(Exception, block)

    def test3(self):
        """
        In this test we will get a list from a particle
        """

        instance = ExampleParticlesInterface()
        self.assertEqual(len(instance.particles), 0)

        theParticle = datamodel.Particle()
        theParticle.mass = 10 | units.kg
        theParticle.x = 0.1 | units.m
        theParticle.y = 0.2 | units.m
        theParticle.z = 0.5 | units.m

        instance.particles.add_particle(theParticle)

        theParticle = datamodel.Particle()
        theParticle.mass = 11 | units.kg
        theParticle.x = 0.1 | units.m
        theParticle.y = 0.2 | units.m
        theParticle.z = 0.5 | units.m

        instance.particles.add_particle(theParticle)

        self.assertEqual(len(instance.particles[0].element_list()), 10)
        list = instance.particles[0].element_list()
        self.assertEqual(list[0].value1, 0 | units.none)
        self.assertEqual(list[0].value2, 0 | units.none)
        self.assertEqual(list[1].value1, 0 | units.none)
        self.assertEqual(list[1].value2, 1 | units.none)
        for x in range(len(list)):
            self.assertEqual(list[x].value1, 0 | units.none)
            self.assertEqual(list[x].value2, x | units.none)

        list = instance.particles[1].element_list()
        for x in range(len(list)):
            self.assertEqual(list[x].value1, 1 | units.none)
            self.assertEqual(list[x].value2, x | units.none)

        # print instance.particles.element_list()
        # print instance.particles.element_list()[0][1].value2

    def test4(self):
        """
        In this test we will get a list from a particle
        """

        instance = ExampleParticlesInterface()
        self.assertEqual(len(instance.particles), 0)

        theParticle = datamodel.Particle()
        theParticle.x = 0.1 | units.m
        theParticle.y = 0.2 | units.m
        theParticle.z = 0.5 | units.m

        self.assertRaises(exceptions.AmuseException, instance.particles.add_particle, theParticle)

    def test5(self):
        """
        In this test we will get subsets from the incode set
        """

        instance = ExampleParticlesInterface()
        self.assertEqual(len(instance.particles), 0)

        particles = datamodel.Particles(10)
        particles.mass = 0.1 | units.kg
        particles.x = 0.1 | units.m
        particles.y = 0.2 | units.m
        particles.z = 0.5 | units.m

        instance.particles.add_particle(particles)

        self.assertEqual(len(instance.particles), 10)

        subset = instance.particles[0:2]
        self.assertEqual(len(subset), 2)
        self.assertTrue(str(subset).find('key') > 0)

    def log(self, message, *arguments):
        print("IN TEST >>", message.format(*arguments))



import os.path

from amuse.test.amusetest import TestWithMPI, get_path_to_results
from amuse.support.exceptions import AmuseException
from amuse.ext.galactics_model import new_galactics_model, new_galactics_gas_model
from amuse.units import nbody_system, generic_unit_converter, constants, units
from amuse.io import write_set_to_file

# testing the gas models is *very* slow


class NewGalactICsModelTests(TestWithMPI):

    def test1(self):
        halo_number_of_particles = 100
        particles = new_galactics_model(halo_number_of_particles, generate_bulge_flag=False,
            generate_disk_flag=False, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=True)

        self.assertEqual(len(particles), halo_number_of_particles)
        self.assertAlmostEqual(particles.total_mass(), 1.0 | nbody_system.mass)
        self.assertAlmostEqual(particles.kinetic_energy(),
            0.25 | nbody_system.energy)

    def test2(self):
        self.assertTrue("GalactICs documentation:" in new_galactics_model.__doc__)
        self.assertTrue("GalactICs allows to generate self-consistent disc-bulge-halo galaxy models." in new_galactics_model.__doc__)

        self.assertTrue("Parameters:" in new_galactics_model.__doc__)
        self.assertTrue("generate_disk_flag" in new_galactics_model.__doc__)
        self.assertTrue("halo_streaming_fraction" in new_galactics_model.__doc__)

        print(new_galactics_model.__doc__)

    def slowtest3(self):
        print("Generate a model for M31, using defaults (100k disk, 50k bulge, 200k halo) - Nbody units")
        halo_number_of_particles = 20000
        particles = new_galactics_model(halo_number_of_particles, do_scale=True,
          bulge_number_of_particles=5000, disk_number_of_particles=10000)

        self.assertEqual(len(particles), 35000)
        self.assertAlmostEqual(particles.total_mass(), 1.0 | nbody_system.mass)
        self.assertAlmostEqual(particles.kinetic_energy(), 0.25 | nbody_system.energy)

        write_set_to_file(particles, os.path.join(get_path_to_results(), 'M31_galactICs.amuse'), 'amuse')

    def slowtest4(self):
        print("Generate a model for a disk galaxy (10k disk, no bulge, 20k halo) - SI units")
        halo_number_of_particles = 20000
        converter = generic_unit_converter.ConvertBetweenGenericAndSiUnits(constants.G, 1.0e12 | units.MSun, 50.0 | units.kpc)
        particles = new_galactics_model(halo_number_of_particles, disk_number_of_particles=10000,
            generate_bulge_flag=False, do_scale=True, unit_system_converter=converter)

        self.assertEqual(len(particles), 30000)
        self.assertAlmostRelativeEquals(particles.total_mass(), 1.0e12 | units.MSun, 10)
        self.assertAlmostRelativeEquals(particles.kinetic_energy(), converter.to_si(0.25 | nbody_system.energy), 10)

        disk = particles[:10000]
        self.assertAlmostRelativeEquals(disk.total_mass(), 2.156e10 | units.MSun, 3)
        self.assertAlmostRelativeEquals(disk.position.lengths_squared().amax().sqrt().in_(units.kpc),
                                                 15.584 | units.kpc, 3)

        self.assertAlmostRelativeEquals(disk.position.std(axis=0).in_(units.kpc), [3.5934, 3.6768, 0.17078] | units.kpc, 3)

        write_set_to_file(particles, os.path.join(get_path_to_results(), 'disk_galactICs.amuse'), 'amuse')

    def test5(self):
        halo_number_of_particles = 1000

        conv = nbody_system.nbody_to_si(1.e12 | units.MSun, 100. | units.kpc)

        particles1 = new_galactics_model(halo_number_of_particles, generate_bulge_flag=False,
            generate_disk_flag=False, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=False)
        particles2 = new_galactics_model(halo_number_of_particles, conv, generate_bulge_flag=False,
            generate_disk_flag=False, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=False)
        particles3 = new_galactics_model(halo_number_of_particles, generate_bulge_flag=False,
            generate_disk_flag=False, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=True)
        particles4 = new_galactics_model(halo_number_of_particles, conv, generate_bulge_flag=False,
            generate_disk_flag=False, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=True)

        self.assertEqual(len(particles1), halo_number_of_particles)
        self.assertEqual(len(particles2), halo_number_of_particles)
        self.assertEqual(len(particles3), halo_number_of_particles)
        self.assertEqual(len(particles4), halo_number_of_particles)

        self.assertAlmostEqual(conv.to_nbody(particles2.total_mass()), particles1.total_mass())
        self.assertAlmostEqual(conv.to_nbody(particles4.total_mass()), particles3.total_mass())
        self.assertAlmostEqual(particles3.total_mass(), 1. | nbody_system.mass)
        self.assertAlmostRelativeEquals(particles4.total_mass(), conv.to_si(1. | nbody_system.mass), 12)

        r1 = particles1.position.lengths().std()
        r2 = particles2.position.lengths().std()
        r3 = particles3.position.lengths().std()
        r4 = particles4.position.lengths().std()
        self.assertAlmostEqual(conv.to_nbody(r2), r1)
        self.assertAlmostEqual(conv.to_nbody(r4), r3)
        self.assertTrue(r1/r3 > 100)  # for the default parameters the scaling is quite drastic
        self.assertTrue(r2/r4 > 100)
        print(r1, r3)
        print(r2, r4)

    def test6(self):
        halo_number_of_particles = 1000

        conv = nbody_system.nbody_to_si(1.e12 | units.MSun, 100. | units.kpc)

        particles1 = new_galactics_gas_model(halo_number_of_particles, bulge_type_parameter=0,
            disk_type_parameter=0, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=False, reuse_cached_model=False, verbose=True)
        particles2 = new_galactics_gas_model(halo_number_of_particles, conv, bulge_type_parameter=0,
            disk_type_parameter=0, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=False, verbose=True)
        particles3 = new_galactics_gas_model(halo_number_of_particles, bulge_type_parameter=0,
            disk_type_parameter=0, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=True, verbose=True)
        particles4 = new_galactics_gas_model(halo_number_of_particles, conv, bulge_type_parameter=0,
            disk_type_parameter=0, order_of_multipole_expansion=0, halo_random_seed=-1,
            do_scale=True, verbose=True)

        self.assertEqual(len(particles1), halo_number_of_particles)
        self.assertEqual(len(particles2), halo_number_of_particles)
        self.assertEqual(len(particles3), halo_number_of_particles)
        self.assertEqual(len(particles4), halo_number_of_particles)

        self.assertAlmostEqual(conv.to_nbody(particles2.total_mass()), particles1.total_mass())
        self.assertAlmostEqual(conv.to_nbody(particles4.total_mass()), particles3.total_mass())
        self.assertAlmostEqual(particles3.total_mass(), 1. | nbody_system.mass)
        self.assertAlmostRelativeEquals(particles4.total_mass(), conv.to_si(1. | nbody_system.mass), 12)

        r1 = particles1.position.lengths().std()
        r2 = particles2.position.lengths().std()
        r3 = particles3.position.lengths().std()
        r4 = particles4.position.lengths().std()
        self.assertAlmostEqual(conv.to_nbody(r2), r1)
        self.assertAlmostEqual(conv.to_nbody(r4), r3)
        self.assertTrue(r1/r3 > 30)  # for the default parameters the scaling is quite drastic
        self.assertTrue(r2/r4 > 30)
        print(r1, r3)
        print(r2, r4)

        # ~ self.assertAlmostEquals(particles.kinetic_energy(),
            # ~ 0.25 | nbody_system.energy)



from amuse.test import amusetest
import os.path
import shutil
import subprocess
import sys

from amuse.rfi.tools import create_dir


class CreateADirectoryAndPopulateItWithFilesForACommunityCodeTest(amusetest.TestWithMPI):

    def setUp(self):

        self.check_can_compile_modules()

    def test1(self):
        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForACCode()
        instance.name_of_the_code_interface_class = 'TestCode'
        self.assertEqual(instance.name_of_the_community_code, 'testcode')
        self.assertTrue(instance.path_of_the_community_code.endswith('testcode'))

    def test2(self):
        root = self.get_path_to_results()
        working_dir = os.path.join(root, 'testcode')

        print(working_dir)

        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)

        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForACCode()
        instance.name_of_the_code_interface_class = 'TestCode'
        instance.path_of_the_root_directory = root
        # ~ if create_dir.CreateADirectoryAndPopulateItWithFiles().amuse_root_dir == os.getcwd():
            # ~ self.assertEqual(instance.reference_to_amuse_path,'../..')

        instance.start()

        self.assertTrue(os.path.exists(working_dir))
        self.assertTrue(os.path.exists(os.path.join(working_dir, 'interface.py')))
        self.assertTrue(os.path.exists(os.path.join(working_dir, 'Makefile')))

        call = subprocess.Popen(
            'make',
            cwd=working_dir,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()

        self.assertEqual(call.returncode, 0)

        self.assertTrue(os.path.exists(os.path.join(working_dir, 'testcode_worker')))

        sys.path.insert(0, root)

        try:
            __import__('testcode.interface')
        except:
            self.fail("import of code failed")

        module = sys.modules['testcode.interface']
        instance = module.TestCode()
        result = instance.echo_int(12)

        self.assertEqual(result, 12)

        instance.stop()

        del sys.path[0]

        self.assertTrue(os.path.exists(os.path.join(working_dir, 'worker_code.cc')))
        call = subprocess.Popen(
            ['make', 'clean'],
            cwd=working_dir,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()

        print(errorstring)
        self.assertFalse(os.path.exists(os.path.join(working_dir, 'worker_code.cc')))

        self.assertEqual(call.returncode, 0)

        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)

    def test3(self):
        root = self.get_path_to_results()
        working_dir = os.path.join(root, 'testcodef')

        print(working_dir)

        if os.path.exists(working_dir):
            shutil.rmtree(working_dir)

        instance = create_dir.CreateADirectoryAndPopulateItWithFilesForAFortranCode()
        instance.name_of_the_code_interface_class = 'TestCodeF'
        instance.path_of_the_root_directory = root
        # ~ if create_dir.CreateADirectoryAndPopulateItWithFiles().amuse_root_dir == os.getcwd():
            # ~ self.assertEqual(instance.reference_to_amuse_path,'../..')

        instance.start()

        self.assertTrue(os.path.exists(working_dir))
        self.assertTrue(os.path.exists(os.path.join(working_dir, 'interface.py')))
        self.assertTrue(os.path.exists(os.path.join(working_dir, 'Makefile')))

        call = subprocess.Popen(
            'make',
            cwd=working_dir,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()

        print(errorstring)

        self.assertEqual(call.returncode, 0)

        self.assertTrue(os.path.exists(os.path.join(working_dir, 'testcodef_worker')))

        sys.path.insert(0, root)

        try:
            __import__('testcodef.interface')
        except:
            self.fail("import of code failed")

        module = sys.modules['testcodef.interface']
        instance = module.TestCodeF()
        result = instance.echo_int(12)

        self.assertEqual(result, 12)

        instance.stop()

        del sys.path[0]

        self.assertTrue(os.path.exists(os.path.join(working_dir, 'worker_code.f90')))

        call = subprocess.Popen(
            ['make', 'clean'],
            cwd=working_dir,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )
        outputstring, errorstring = call.communicate()
        print(errorstring)

        self.assertFalse(os.path.exists(os.path.join(working_dir, 'worker_code.f90')))

        self.assertEqual(call.returncode, 0)

   #     if os.path.exists(working_dir):
        #    shutil.rmtree(working_dir)



import numpy

from amuse.test.amusetest import TestWithMPI
from amuse.units import nbody_system
from amuse.units import units
from amuse import datamodel
from amuse.community.higpus.interface import HiGPUsInterface, HiGPUs
from amuse.ic.plummer import new_plummer_model


class HiGPUsInterfaceTests(TestWithMPI):

    def test0(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.cleanup_code()
        instance.stop()

    def test1(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(32)
        instance.set_number_of_GPU(1)
        error = instance.commit_parameters()
        index, error = instance.new_particle(1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
        self.assertEqual(error, 0)
        self.assertEqual(index, 0)
        index, error = instance.new_particle(0.000003003, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0)
        self.assertEqual(error, 0)
        self.assertEqual(index, 1)
        error = instance.commit_particles()
        self.assertEqual(error, 0)
        retrieved_state = instance.get_state(index)
        self.assertEqual(retrieved_state['__result'], 0)
        self.assertEqual(0.000003003,  retrieved_state['mass'])
        self.assertEqual(0.0, retrieved_state['radius'])
        self.assertEqual(instance.get_number_of_particles()['number_of_particles'], 2)
        instance.cleanup_code()
        instance.stop()

    def test2(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        for x in [0.101, 4.0]:
            error = instance.set_eta4(x)
            self.assertEqual(error, 0)
            value, error = instance.get_eta4()
            self.assertEqual(error, 0)
            self.assertEqual(x, value)
        instance.cleanup_code()
        instance.stop()

    def test3(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(32)
        instance.set_number_of_GPU(1)
        error = instance.commit_parameters()

        instance.new_particle([11.0, 12.0, 13.0, 14.0],
            [2.0, 3.0, 4.0, 5.0],
            [2.1, 3.1, 4.1, 5.1],
            [2.2, 3.2, 4.2, 5.2],
            [2.3, 3.3, 4.3, 5.3],
            [2.4, 3.4, 4.4, 5.4],
            [2.5, 3.5, 4.5, 5.5],
            [2.6, 3.6, 4.6, 5.6])
        error = instance.commit_particles()
        retrieved_state = instance.get_state(0)
        self.assertEqual(11.0,  retrieved_state['mass'])
        retrieved_state = instance.get_state([2, 3, 4])
        self.assertEqual(14.0,  retrieved_state['mass'][1])
        self.assertEqual(instance.get_number_of_particles()['number_of_particles'], 4)
        instance.cleanup_code()
        instance.stop()

    def test4(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(128)
        instance.set_number_of_GPU(1)
        error = instance.commit_parameters()

        n = 4000
        ids = [i for i in range(1, n)]
        values = [1.0 * i for i in range(1, n)]
        instance.new_particle(values, values, values, values, values, values, values)
        error = instance.commit_particles()
        retrieved_state = instance.get_state(0)
        self.assertEqual(1.0,  retrieved_state['mass'])
        retrieved_state = instance.get_state(3998)
        self.assertEqual(3999.0,  retrieved_state['mass'])
        instance.cleanup_code()
        instance.stop()

    def test5(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(32)
        instance.set_number_of_GPU(1)
        instance.set_gpu_name("GeForce GTX 480")
        instance.set_eps(0.0)
        error = instance.commit_parameters()

        instance.new_particle(
            [1.0, 1.0, 1.0],
            [0.0, 0.0, 0.0],
            [1.0, 0.0, -1.0],
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0],
            [0.0, 1.0, 0.0],
            [0.0, 0.0, 0.0],
            [0.0, 0.0, 0.0])

        instance.commit_particles()
        Ep = instance.get_potential_energy()['potential_energy']
        Ek = instance.get_kinetic_energy()['kinetic_energy']
        self.assertEqual(Ek, 0.5)
        self.assertEqual(Ep, -2.5)
        instance.delete_particle(1)
        instance.recommit_particles()
        n = instance.get_number_of_particles()['number_of_particles']
        Ep = instance.get_potential_energy()['potential_energy']
        Ek = instance.get_kinetic_energy()['kinetic_energy']

        self.assertEqual(n, 2)
        self.assertEqual(Ek, 0.)
        self.assertEqual(Ep, -0.5)
        instance.cleanup_code()
        instance.stop()

    def test6(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(32)
        instance.set_number_of_GPU(1)
        instance.set_gpu_name("GeForce GTX 480")
        instance.set_eps(0.0)
        error = instance.commit_parameters()

        id1, errorcode = instance.new_particle(mass=10.0, radius=1.0, x=0.0, y=0.0, z=0.0, vx=0.0, vy=0.0, vz=0.0)
        id2, errorcode = instance.new_particle(mass=10.0, radius=1.0, x=2.0, y=0.0, z=0.0, vx=10.0, vy=0.0, vz=0.0)

        instance.commit_particles()
        potential, errorcode = instance.get_potential(id1)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -10.0 / (2.0**2 + 0.1**2)**0.5, 2)
        total_potential, errorcode = instance.get_potential_energy()
        potentials, errorcode = instance.get_potential([id1, id2])

        self.assertAlmostRelativeEquals(total_potential, numpy.sum(potentials * 10.0) / 2.0)
        instance.cleanup_code()
        instance.stop()

    def test7(self):
        instance = self.new_instance_of_an_optional_code(HiGPUsInterface)
        instance.initialize_code()
        instance.set_number_of_Threads(32)
        instance.set_number_of_GPU(1)
        instance.set_gpu_name("GeForce GTX 480")
        instance.set_eps(0.0)
        error = instance.commit_parameters()

        id1, errorcode = instance.new_particle(mass=10.0, radius=1.0, x=0.0, y=0.0, z=0.0, vx=0.0, vy=0.0, vz=0.0)
        id2, errorcode = instance.new_particle(mass=1.0, radius=1.0, x=2.0, y=0.0, z=0.0, vx=10.0, vy=0.0, vz=0.0)

        instance.commit_particles()
        potential, errorcode = instance.get_potential(id1)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -1.0 / numpy.sqrt(2.0**2), 8)

        potential, errorcode = instance.get_potential(id2)
        self.assertEqual(errorcode, 0)
        self.assertAlmostRelativeEquals(potential,  -10.0 / numpy.sqrt(2.0**2), 8)

        total_potential, errorcode = instance.get_potential_energy()
        potentials, errorcode = instance.get_potential([id1, id2])

        self.assertAlmostRelativeEquals(total_potential, numpy.sum(potentials * [10.0, 1.0]) / 2.0)
        instance.cleanup_code()
        instance.stop()


class TestHiGPUs(TestWithMPI):
    def new_system_of_sun_and_earth(self):
        stars = datamodel.Stars(2)
        sun = stars[0]
        sun.mass = units.MSun(1.0)
        sun.position = units.m(numpy.array((0.0, 0.0, 0.0)))
        sun.velocity = units.ms(numpy.array((0.0, 0.0, 0.0)))
        sun.radius = units.RSun(1.0)

        earth = stars[1]
        earth.mass = units.kg(5.9736e24)
        earth.radius = units.km(6371.)
        earth.position = units.km(numpy.array((149.5e6, 0.0, 0.0)))
        earth.velocity = units.ms(numpy.array((0.0, 29800., 0.0)))

        return stars

    def test1(self):
        convert_nbody = nbody_system.nbody_to_si(1.0 | units.MSun, 149.5e6 | units.km)
        instance = self.new_instance_of_an_optional_code(HiGPUs, convert_nbody)

        stars = self.new_system_of_sun_and_earth()
        earth = stars[1]

        instance.particles.add_particles(stars)
        instance.evolve_model(365 | units.day)

        instance.particles.copy_values_of_all_attributes_to(stars)

        position_at_start = earth.position.value_in(units.AU)[0]
        position_after_full_rotation = earth.position.value_in(units.AU)[0]
        self.assertAlmostEqual(position_at_start, position_after_full_rotation, 6)

        instance.evolve_model(365.0 + (365.0 / 2) | units.day)

        instance.particles.copy_values_of_all_attributes_to(stars)
        position_after_half_a_rotation = earth.position.value_in(units.AU)[0]
        self.assertAlmostEqual(-position_at_start, position_after_half_a_rotation, 2)

        instance.evolve_model(365.0 + (365.0 / 2) + (365.0 / 4) | units.day)

        instance.particles.copy_values_of_all_attributes_to(stars)
        position_after_half_a_rotation = earth.position.value_in(units.AU)[1]
        self.assertAlmostEqual(-position_at_start, position_after_half_a_rotation, 2)

        instance.stop()

    def test2(self):
        convert_nbody = nbody_system.nbody_to_si(5.0 | units.kg, 10.0 | units.m)

        instance = self.new_instance_of_an_optional_code(HiGPUs, convert_nbody)

        instance.initialize_code()
        instance.commit_parameters()

        particles = datamodel.Particles(2)
        self.assertEqual(len(instance.particles), 0)

        particles.mass = [15.0, 30.0] | units.kg
        particles.radius = [10.0, 20.0] | units.m
        particles.position = [[10.0, 20.0, 30.0], [20.0, 40.0, 60.0]] | units.m
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | units.m / units.s

        instance.particles.add_particles(particles)
        self.assertEqual(len(instance.particles), 2)

        instance.particles.mass = [17.0, 33.0] | units.kg

        self.assertEqual(instance.get_mass(0), 17.0 | units.kg)
        self.assertEqual(instance.get_mass(1), 33.0 | units.kg)

        instance.cleanup_code()
        instance.stop()

    def test3(self):
        instance = self.new_instance_of_an_optional_code(HiGPUs)
        instance.initialize_code()
        instance.parameters.eps = 0.0 | nbody_system.length
        instance.commit_parameters()
        particles = datamodel.Particles(4)
        particles.mass = nbody_system.mass.new_quantity(range(1, 5))
        particles.radius = 0.00001 | nbody_system.length
        particles.position = [[-1.0, 0.0, 0.0], [1.0, 0.0, 0.0], [0.0, -1.0, 0.0], [0.0, 1.0, 0.0]] | nbody_system.length
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | nbody_system.speed

        instance.particles.add_particles(particles)

        instance.commit_particles()
        copyof = instance.particles.copy()

        instance.cleanup_code()
        instance.stop()

        self.assertEqual(2 | nbody_system.mass, copyof[1].mass)

    def test4(self):
        self.skip("Need to implement get_gravity_at_point for HiGPUs")
        instance = self.new_instance_of_an_optional_code(HiGPUs)
        instance.initialize_code()
        instance.parameters.eps = 0.0
        instance.commit_parameters()

        particles = datamodel.Particles(2)
        particles.mass = [1.0, 1.0] | nbody_system.mass
        particles.radius = [0.0001, 0.0001] | nbody_system.length
        particles.position = [[0.0, 0.0, 0.0], [2.0, 0.0, 0.0]] | nbody_system.length
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 0.0, 0.0]] | nbody_system.speed
        instance.particles.add_particles(particles)

        instance.commit_particles()

        zero = 0.0 | nbody_system.length
        fx, fy, fz = instance.get_gravity_at_point(0.0000001 | nbody_system.length, 2.0 | nbody_system.length, zero, zero)
        self.assertAlmostEqual(fx, -0.25 | nbody_system.acceleration)
        self.assertAlmostEqual(fy, 0.0 | nbody_system.acceleration)
        self.assertAlmostEqual(fz, 0.0 | nbody_system.acceleration)
        for x in (0.25, 0.5, 0.75):
            x0 = x | nbody_system.length
            x1 = (2.0 - x) | nbody_system.length
            potential0 = instance.get_potential_at_point(0.0000001 | nbody_system.length, x0, zero, zero)
            potential1 = instance.get_potential_at_point(0.0000001 | nbody_system.length, x1, zero, zero)
            fx0, fy0, fz0 = instance.get_gravity_at_point(0.0000001 | nbody_system.length, x0, zero, zero)
            fx1, fy1, fz1 = instance.get_gravity_at_point(0.0000001 | nbody_system.length, x1, zero, zero)

            self.assertAlmostEqual(fy0, 0.0 | nbody_system.acceleration)
            self.assertAlmostEqual(fz0, 0.0 | nbody_system.acceleration)
            self.assertAlmostEqual(fy1, 0.0 | nbody_system.acceleration)
            self.assertAlmostEqual(fz1, 0.0 | nbody_system.acceleration)

            self.assertAlmostEqual(fx0, -1.0 * fx1, 5)
            fx = (-1.0 / (x0**2) + 1.0 / (x1**2)) * (1.0 | nbody_system.length ** 3 / nbody_system.time ** 2)
            self.assertAlmostEqual(fx, fx0, 2)
            self.assertAlmostEqual(potential0, potential1, 5)

        instance.cleanup_code()
        instance.stop()

    def test5(self):
        instance = self.new_instance_of_an_optional_code(HiGPUs)
        instance.initialize_code()

        instance.parameters.eta6 = 0.01
        instance.commit_parameters()

        stars = new_plummer_model(64)

        instance.particles.add_particles(stars)
        instance.commit_particles()

        instance.evolve_model(0.001 | nbody_system.time)

        e0 = instance.get_kinetic_energy() + instance.get_potential_energy()

        stars.mass *= 0.9

        instance.synchronize_model()

        e1 = instance.get_kinetic_energy() + instance.get_potential_energy()

        instance.cleanup_code()
        instance.stop()

        delta_e = e1 - e0

        self.assertTrue(e1 != e0)

    def test6(self):
        particles = datamodel.Particles(2)
        particles.mass = [1.0, 0.000003003] | nbody_system.mass
        particles.radius = [0.0001, 0.0001] | nbody_system.length
        particles.position = [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0]] | nbody_system.length
        particles.velocity = [[0.0, 0.0, 0.0], [0.0, 1.0, 0.0]] | nbody_system.speed

        particle = datamodel.Particles(1)
        particle.mass = 0.000003003 | nbody_system.mass
        particle.radius = 0.0001 | nbody_system.length
        particle.position = [0.0, 1.0, 0.0] | nbody_system.length
        particle.velocity = [-1.0, 0.0, 0.0] | nbody_system.speed

        instance = self.new_instance_of_an_optional_code(HiGPUs)
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.initialize_code()
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        instance.commit_parameters()
        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')
        instance.particles.add_particles(particles)
        instance.commit_particles()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        instance.evolve_model(1 | nbody_system.time)
        self.assertEqual(instance.get_name_of_current_state(), 'EVOLVED')
        instance.particles.remove_particle(particles[1])
        instance.particles.add_particle(particle)
        self.assertEqual(instance.get_name_of_current_state(), 'UPDATE')
        instance.recommit_particles()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        instance.evolve_model(1 | nbody_system.time)
        self.assertEqual(instance.get_name_of_current_state(), 'EVOLVED')
        instance.synchronize_model()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        instance.cleanup_code()
        self.assertEqual(instance.get_name_of_current_state(), 'END')
        instance.stop()

    def test7(self):

        instance = self.new_instance_of_an_optional_code(HiGPUs)
        instance.parameters.eta6 = 0.5

        instance.commit_parameters()

        self.assertEqual(instance.parameters.eta6,  0.5)
        self.assertEqual(instance.parameters.eta4,  0.01)
        self.assertEqual(instance.parameters.begin_time,  0.0 | nbody_system.time)
        self.assertEqual(instance.parameters.r_core_plummer,  0.0 | nbody_system.length)
        self.assertEqual(instance.parameters.mass_plummer,  0.0 | nbody_system.mass)
        self.assertEqual(instance.parameters.Threads,  128)
        self.assertEqual(instance.parameters.n_Print,  1000000)
        self.assertEqual(instance.parameters.dt_Print,  1000000.0 | nbody_system.time)
        self.assertEqual(instance.parameters.max_step,  pow(2., -3.0) | nbody_system.time)
        self.assertEqual(instance.parameters.min_step,  pow(2., -30.0) | nbody_system.time)
        self.assertEqual(instance.parameters.gpu_name,  "")
        self.assertEqual(instance.parameters.n_gpu,  1)
        instance.cleanup_code()
        instance.stop()

    def test8(self):

        instance = self.new_instance_of_an_optional_code(HiGPUs)

        particles = new_plummer_model(128)

        instance.particles.add_particles(particles)

        E0 = instance.kinetic_energy+instance.potential_energy

        instance.evolve_model(1. | nbody_system.time)

        E1 = instance.kinetic_energy+instance.potential_energy

        self.assertLess(abs(E1-E0)/abs(E0), 1.e-6)

        instance.stop()

    def test9(self):

        instance = self.new_instance_of_an_optional_code(HiGPUs)

        particles = new_plummer_model(128)

        particles[0:64].mass *= 0
        particles[64:].mass *= 2

        instance.particles.add_particles(particles)

        E0 = instance.kinetic_energy+instance.potential_energy

        instance.evolve_model(1. | nbody_system.time)

        E1 = instance.kinetic_energy+instance.potential_energy

        self.assertLess(abs(E1-E0)/abs(E0), 1.e-6)

        instance.stop()



import numpy

from amuse.units import units, nbody_system
from amuse.units.quantities import zero
from amuse.datamodel import Particles, Particle
from amuse.support.exceptions import AmuseException
from amuse.test.amusetest import TestWithMPI
from amuse.plot import pynbody_column_density_plot, HAS_PYNBODY

from amuse.community.bhtree.interface import BHTree
from amuse.community.gadget2.interface import Gadget2
from amuse.community.seba.interface import SeBa
from amuse.community.evtwin.interface import EVtwin

from amuse.ext.plotting_hydro import new_plotting_hydrodynamics_code
from amuse.ext.hydro_collision import StellarEncounterInHydrodynamics


class TestStellarEncounterInHydrodynamics(TestWithMPI):

    def new_colliders(self):
        colliders = Particles(2)
        colliders.mass = [5, 2] | units.MSun
        colliders.position = [[0.0, 0.0, 0.0], [1.0, 0.0, 0.0]] | units.RSun
        colliders.velocity = [[0.0, 0.0, 0.0], [0.0, 2000.0, 0.0]] | units.km / units.s
        colliders.move_to_center()
        return colliders

    def test1(self):
        print("Test collect_required_attributes")
        in_memory = self.new_colliders()
        gravity = BHTree(nbody_system.nbody_to_si(1 | units.MSun, 1.0 | units.RSun))
        gravity.particles.add_particles(in_memory)
        stellar = SeBa()
        stellar.particles.add_particles(in_memory)

        collision = StellarEncounterInHydrodynamics(None, None, verbose=True)

        self.assertFalse(hasattr(in_memory, "radius"))
        collision.collect_required_attributes(in_memory, gravity, stellar)
        self.assertTrue(hasattr(in_memory, "radius"))
        self.assertAlmostRelativeEqual(in_memory.radius.sum(), 4.2458 | units.RSun, 3)

        from_stellar = stellar.particles.copy()
        for attribute in ["x", "y", "z", "vx", "vy", "vz"]:
            self.assertFalse(hasattr(from_stellar, attribute))
        collision.collect_required_attributes(from_stellar, gravity, stellar)

        gravity.stop()
        stellar.stop()

        for attribute in ["x", "y", "z", "vx", "vy", "vz"]:
            self.assertTrue(hasattr(from_stellar, attribute))
        self.assertAlmostEqual(from_stellar.position, in_memory.position)
        self.assertAlmostEqual(from_stellar.velocity, in_memory.velocity)

    def test2(self):
        print("Test backtrack_particles")
        colliders = self.new_colliders()
        colliders.radius = [1, 2] | units.RSun

        collision = StellarEncounterInHydrodynamics(None, None, verbose=True)

        relative_position = colliders[1].position - colliders[0].position
        self.assertAlmostRelativeEqual(relative_position.length(), 1.0 | units.RSun, 7)

        total_energy_before = colliders.kinetic_energy() + colliders.potential_energy()
        self.assertTrue(total_energy_before > zero)

        collision.backtrack_particles(colliders)
        collision.kepler.stop()

        relative_position = colliders[1].position - colliders[0].position
        self.assertAlmostRelativeEqual(relative_position.length(), 15 | units.RSun, 3)

        total_energy_after = colliders.kinetic_energy() + colliders.potential_energy()
        self.assertAlmostRelativeEqual(total_energy_after, total_energy_before, 7)

    def test3(self):
        print("Test convert_stars")
        colliders = self.new_colliders()
        colliders.position = [[-100.0, 0.0, 0.0], [100.0, 0.0, 0.0]] | units.RSun
        stellar = EVtwin()
        stellar.particles.add_particles(colliders)

        collision = StellarEncounterInHydrodynamics(700, None, relax_sph_models=False, verbose=True)
        gas_particles = collision.convert_stars(colliders, stellar)
        stellar.stop()

        self.assertEqual(gas_particles.mass, 0.01 | units.MSun)
        self.assertTrue(numpy.all(gas_particles[:500].x < zero))
        self.assertTrue(numpy.all(gas_particles[500:].x > zero))

        self.assertIsOfOrder((
                gas_particles[:500].position - ([-100.0, 0.0, 0.0] | units.RSun)
            ).lengths_squared().amax().sqrt(), 1 | units.RSun)
        self.assertIsOfOrder((
                gas_particles[500:].position - ([100.0, 0.0, 0.0] | units.RSun)
            ).lengths_squared().amax().sqrt(), 1 | units.RSun)

        self.assertAlmostEqual(gas_particles[500:].center_of_mass_velocity().y -
            gas_particles[:500].center_of_mass_velocity().y, 2000.0 | units.km / units.s)

    def test4(self):
        print("Test binary_will_collide")
        collision = StellarEncounterInHydrodynamics(None, None, verbose=True)
        collision.dynamical_timescale = zero
        collision.start_kepler(7 | units.MSun, 10 | units.RSun)
        # at periastron, close enough:
        colliders = self.new_colliders()
        colliders.radius = 1 | units.RSun
        self.assertTrue(collision.binary_will_collide(colliders[0], colliders[1]))
        # at periastron, distance too large:
        colliders.radius = 0.4 | units.RSun
        self.assertFalse(collision.binary_will_collide(colliders[0], colliders[1]))
        # at apastron, will collide at periastron:
        colliders.velocity = [[0.0, 0.0, 0.0], [0.0, 1000.0, 0.0]] | units.km / units.s
        self.assertTrue(collision.binary_will_collide(colliders[0], colliders[1]))
        # hyperbolic orbits, moving away from each other:
        colliders.position = [[0.0, 0.0, 0.0], [1.0, 100.0, 0.0]] | units.RSun
        self.assertFalse(collision.binary_will_collide(colliders[0], colliders[1]))
        # hyperbolic orbits, moving towards each other:
        colliders.velocity = [[0.0, 0.0, 0.0], [0.0, -1000.0, 0.0]] | units.km / units.s
        self.assertTrue(collision.binary_will_collide(colliders[0], colliders[1]))
        collision.kepler.stop()

    def test5(self):
        print("Test group_bound_particles")
        colliders = self.new_colliders()
        colliders.position = [[0.0, 0.0, 0.0], [1.1, 0.0, 0.0]] | units.RSun
        colliders.velocity = [[0.0, 0.0, 0.0], [10000, 0.0, 0.0]] | units.km / units.s
        stellar = EVtwin()
        stellar.particles.add_particles(colliders)

        collision = StellarEncounterInHydrodynamics(7000, None, relax_sph_models=False, verbose=True,
            star_to_sph_arguments=dict(base_grid_options=dict(type="sobol")))
        collision.dynamical_timescale = zero
        gas_particles = collision.convert_stars(colliders, stellar)
        stellar.stop()

        collision.hop = collision.new_hop(gas_particles)
        collision.start_kepler(7 | units.MSun, 10 | units.RSun)
        self.assertTrue(collision.encounter_is_over(gas_particles))
        collision.hop.stop()
        collision.kepler.stop()

        groups = collision.groups_after_encounter
        self.assertEqual(len(groups), 2)
        self.assertTrue(4500 < len(groups[0]) < 5000)
        self.assertTrue(1800 < len(groups[1]) < 2000)
        self.assertEqual(len(gas_particles - groups[0] - groups[1]), 346)
        self.assertAlmostEqual(groups[0].center_of_mass()[0], 0 | units.RSun, 1)
        self.assertAlmostEqual(groups[1].center_of_mass()[0], 1.1 | units.RSun, 0)
        self.assertIsOfOrder(groups[1].center_of_mass_velocity()[0], 10000 | units.km / units.s)

    def test6(self):
        print("Test handle_collision")
        position_offset = [100.0, 200.0, 300.0] | units.RSun
        velocity_offset = [10000.0, 20000.0, 30000.0] | units.km / units.s
        colliders = self.new_colliders()
        colliders.position *= 1.5  # Grazing collision
        colliders.position += position_offset
        colliders.velocity += velocity_offset

        class GravityCodeStub(object):
            def __init__(self, particles):
                self.particles = particles
        gravity = GravityCodeStub(colliders)

        stellar = EVtwin()
        stellar.particles.add_particles(colliders)

        collision = StellarEncounterInHydrodynamics(280,  # For speed
            Gadget2,
            relax_sph_models=False,  # For speed
            verbose=True)
        collision.extra_steps_when_encounter_is_over = 0  # For speed, no extra steps...
        collision.dynamical_timescales_per_step = 1.3  # ... but then we need to evolve a bit longer in the first step
        result = collision.handle_collision(colliders[0], colliders[1], gravity_code=gravity, stellar_evolution_code=stellar)
        stellar.stop()
        print(result)
        self.assertTrue(isinstance(result, Particles))
        self.assertEqual(len(result), 2)
        self.assertAlmostEqual(result.mass, [5.0, 2.0] | units.MSun, 1)
        self.assertAlmostRelativeEqual(result.center_of_mass(), position_offset, 2)
        self.assertAlmostRelativeEqual(result.center_of_mass_velocity(), velocity_offset, 2)

    def slowtest7(self):
        print("Test handle_collision")
        position_offset = [100.0, 200.0, 300.0] | units.RSun
        velocity_offset = [10000.0, 20000.0, 30000.0] | units.km / units.s
        colliders = self.new_colliders()
        colliders.position += position_offset
        colliders.velocity += velocity_offset

        class GravityCodeStub(object):
            def __init__(self, particles):
                self.particles = particles
        gravity = GravityCodeStub(colliders)

        stellar = EVtwin()
        stellar.particles.add_particles(colliders)

        collision = StellarEncounterInHydrodynamics(
            # 28000, # Bit too slow, even for a slowtest...
            2800,
            new_plotting_hydrodynamics_code(Gadget2, 0.2 | units.hour,
                plot_function=pynbody_column_density_plot if HAS_PYNBODY else None,
                plot_function_arguments=dict(width=20 | units.RSun, vmin=29, vmax=35) if HAS_PYNBODY else dict(width=20 | units.RSun)),
            hydrodynamics_arguments=dict(redirection="file", redirect_file="hydro_collision_slowtest7_gadget_out.log"),
            hydrodynamics_parameters=dict(time_limit_cpu=1 | units.day, gas_epsilon=0.01 | units.RSun),
            verbose=True,
            debug=False
        )
        result = collision.handle_collision(colliders[0], colliders[1], gravity_code=gravity, stellar_evolution_code=stellar)
        stellar.stop()
        print(result)
        self.assertTrue(isinstance(result, Particles))
        self.assertEqual(len(result), 2)
        self.assertTrue((result.mass < [5.0, 2.0] | units.MSun).all())
        self.assertAlmostRelativeEqual(result.center_of_mass(), position_offset, 2)
        self.assertAlmostRelativeEqual(result.center_of_mass_velocity(), velocity_offset, 2)



from amuse.test.amusetest import TestWithMPI

from amuse.community.bhtree.interface import BHTree
from amuse.community.hermite.interface import Hermite
from amuse.community.phigrape.interface import PhiGRAPE
from amuse.community.gadget2.interface import Gadget2
from amuse.community.fi.interface import Fi
from amuse.community.ph4.interface import ph4
from amuse.community.mi6.interface import MI6

import numpy
import time
from amuse.units import units
from amuse.units import nbody_system
from amuse import datamodel
from amuse.ic.plummer import new_plummer_model


class _TestGravityCodes(TestWithMPI):
    length_unit = nbody_system.length
    speed_unit = nbody_system.speed
    mass_unit = nbody_system.mass
    time_unit = nbody_system.time

    @property
    def nbody_converter(self):
        return None

    def gravity_code_factory(self):
        self.skip("abstract test")

    def test1(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        particles = new_plummer_model(100, convert_nbody=self.nbody_converter)
        particles.radius = 0 | self.length_unit
        particles.move_to_center()
        instance.particles.add_particles(particles)
        instance.commit_particles()
        self.assertEqual(len(instance.particles), 100)
        outer = particles.select(lambda r: r.length() > (0.5 | self.length_unit), ["position"])
        print(len(outer))
        self.assertTrue(len(outer) > 0)
        self.assertTrue(len(outer) < 100)
        instance.synchronize_model()
        instance.particles.remove_particles(outer)
        instance.recommit_particles()
        self.assertEqual(len(instance.particles), 100-len(outer))
        number_of_particles_in_module = instance.legacy_interface.get_number_of_particles()['number_of_particles']
        self.assertEqual(number_of_particles_in_module, 100-len(outer))
        instance.stop()

    def getset_attribute(self, attributename, attributevalue1, attributevalue2):

        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        try:
            particles = new_plummer_model(100)
            particles.move_to_center()
            particles.radius = 0.0 | nbody_system.length
            instance.particles.add_particles(particles)
            instance.commit_particles()
            setattr(instance.particles, attributename, attributevalue1)
            actual = getattr(instance.particles, attributename)
            self.assertAlmostRelativeEqual(actual, attributevalue1)
            setattr(instance.particles, attributename, attributevalue2)
            actual = getattr(instance.particles, attributename)
            print(actual.as_quantity_in(attributevalue2.unit))
            self.assertAlmostRelativeEqual(actual, attributevalue2)
        finally:
            instance.stop()

    def test2(self):
        self.getset_attribute("radius", 1.0 | self.length_unit, 2.0 | self.length_unit)

    def test3(self):
        self.getset_attribute("position", [1.0, 2.0, 3.0] | self.length_unit, [5.0, 6.0, 7.0] | self.length_unit)

    def test4(self):
        self.getset_attribute("velocity", [1.0, 2.0, 3.0] | self.speed_unit, [5.0, 6.0, 7.0] | self.speed_unit)

    def test5(self):
        self.getset_attribute("mass", 1.0 | self.mass_unit, 2.0 | self.mass_unit)

    def test6(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        try:
            particles = new_plummer_model(100, convert_nbody=self.nbody_converter)
            more_particles = new_plummer_model(50, convert_nbody=self.nbody_converter)
            particles.radius = 0 | self.length_unit
            more_particles.radius = 1.0 | self.length_unit
            particles.move_to_center()
            more_particles.move_to_center()

            instance.particles.add_particles(particles)
            instance.commit_particles()
            self.assertEqual(len(instance.particles), 100)
            instance.synchronize_model()
            instance.particles.add_particles(more_particles)
            instance.recommit_particles()
            number_of_particles_in_module = instance.legacy_interface.get_number_of_particles()['number_of_particles']
            self.assertEqual(len(instance.particles), 150)
            self.assertEqual(number_of_particles_in_module, 150)
            instance.synchronize_model()
            instance.particles.remove_particles(particles)
            self.assertEqual(len(instance.particles), 50)
            instance.recommit_particles()
            instance.synchronize_model()
            number_of_particles_in_module = instance.legacy_interface.get_number_of_particles()['number_of_particles']
            self.assertEqual(len(instance.particles), 50)
            self.assertEqual(number_of_particles_in_module, 50)
        finally:
            instance.stop()

    def test7(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        try:
            particles = new_plummer_model(100, convert_nbody=self.nbody_converter)
            new_particles = new_plummer_model(50, convert_nbody=self.nbody_converter)
            instance.particles.add_particles(particles)
            instance.commit_particles()

            self.assertEqual(len(instance.particles), 100)
            instance.synchronize_model()
            instance.particles.remove_particle(particles[40])
            instance.particles.remove_particle(particles[70])
            instance.particles.add_particle(new_particles[0])
            print(new_particles[0].key)
            instance.recommit_particles()
            # test the get_mass, get_position and get_velocity functions
            # if they are implemented for the code, otherwise will call
            # get_state multiple times
            # todo, fi fails, need to check with inti
            # self.assertAlmostRelativeEqual(instance.particles[-1].mass, new_particles[0].mass)
            # self.assertAlmostRelativeEqual(instance.particles[-1].velocity, new_particles[0].velocity)
            # self.assertAlmostRelativeEqual(instance.particles[-1].position, new_particles[0].position)
            instance.particles.synchronize_to(particles)
            self.assertEqual(len(particles), 99)
            self.assertEqual(particles[-1], new_particles[0])
        finally:
            instance.stop()

    def test8(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        random = numpy.random.mtrand.RandomState(3456)
        particles = new_plummer_model(10, convert_nbody=self.nbody_converter, random=random)
        particles.radius = 0.2 | self.length_unit
        particles.move_to_center()
        instance.particles.add_particles(particles)
        self.assertEqual(len(instance.particles), 10)
        collision_detection = instance.stopping_conditions.collision_detection
        collision_detection.enable()
        instance.evolve_model(1 | self.time_unit)
        self.assertTrue(collision_detection.is_set())
        instance.stop()

    def test9(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        instance.parameters.stopping_conditions_out_of_box_size = 2 | self.length_unit
        particles = datamodel.Particles(3)
        particles.x = [0, 1, 2] | self.length_unit
        particles.y = 0 | self.length_unit
        particles.z = 0 | self.length_unit
        particles.vx = [0, 0, 2] | self.speed_unit
        particles.vy = [1, -1, 0] | self.speed_unit
        particles.vz = 0 | self.speed_unit
        particles.mass = 1 | self.mass_unit
        particles.radius = 0.0 | self.length_unit
        instance.particles.add_particles(particles)
        stopping_condition = instance.stopping_conditions.out_of_box_detection
        stopping_condition.enable()
        instance.evolve_model(1 | self.time_unit)
        print(instance.particles)
        print(instance.particles.center_of_mass())
        print((instance.particles.position - instance.particles.center_of_mass()).lengths())
        self.assertTrue(stopping_condition.is_set())
        instance.stop()

    def test10(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory)
        particle = datamodel.Particle()
        particle.position = [0, 0, 0] | self.length_unit
        particle.velocity = [1, -2, 3.0] | self.speed_unit
        particle.mass = 1 | self.mass_unit
        particle.radius = 0.0 | self.length_unit

        instance.particles.add_particle(particle)
        instance.evolve_model(1 | self.time_unit)
        self.assertAlmostEqual(instance.model_time, 1 | self.time_unit)
        self.assertAlmostEqual(instance.kinetic_energy, 7.0 | self.mass_unit * self.speed_unit**2)
        self.assertAlmostEqual(instance.potential_energy, 0.0 | self.mass_unit * self.speed_unit**2)
        self.assertAlmostEqual(instance.particles[0].position, [1.0, -2.0, 3.0] | self.length_unit)
        instance.stop()

    def new_gravity_code(self):
        self.gravity_code_factory()


class TestBHTreeGravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return BHTree

    def test9(self):
        self.skip("no support for out of box detection")


class TestHermiteGravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return Hermite

    def test10(self):
        self.skip("no support for single particle")


class TestPH4GravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return ph4

    def test9(self):
        self.skip("no support for out of box detection")


class TestMI6GravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return MI6

    def test6(self):
        self.skip("MI6 crashes on removal and addition of particles")

    def test7(self):
        self.skip("MI6 crashes on removal and addition of particles")

    def test9(self):
        self.skip("no support for out of box detection")

    def test10(self):
        self.skip("no support for single particle")


class TestPhiGRAPEGravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return PhiGRAPE

    def slowtestextra0(self):
        factory = self.gravity_code_factory()
        instance = self.new_instance_of_an_optional_code(factory, mode='gpu')
        try:
            sc = instance.stopping_conditions.collision_detection

            n = 1000
            instance.parameters.initialize_gpu_once = 1
            sc.enable()

            particles = new_plummer_model(n, convert_nbody=self.nbody_converter)
            particles.radius = 1.0 / n | self.length_unit
            particles.move_to_center()

            instance.particles.add_particles(particles)
            instance.commit_particles()
            self.assertEqual(len(instance.particles), n)
            instance.synchronize_model()
            for t in range(1, 101):
                instance.evolve_model((t * 0.01) | self.time_unit)
                print(sc.is_set())
                if sc.is_set():
                    particle1 = sc.particles(0)[0]
                    particle2 = sc.particles(1)[0]
                    newparticle = datamodel.Particles(1)
                    newparticle.mass = particle1.mass + particle2.mass
                    newparticle.radius = particle2.radius
                    newparticle.position = (particle1.position + particle2.position) / 2
                    newparticle.velocity = (particle1.velocity + particle2.velocity) / 2
                    instance.particles.remove_particle(particle1)
                    instance.particles.remove_particle(particle2)
                    merged = instance.particles.add_particles(newparticle)
                    print('Remnant:\n', merged)

        finally:
            instance.stop()

    def test9(self):
        self.skip("no support for out of box detection")


class TestFiGravityCode(_TestGravityCodes):

    def gravity_code_factory(self):
        return Fi

    def test8(self):
        self.skip("no support for collision detection")

    def test9(self):
        self.skip("no support for out of box detection")

    def test10(self):
        self.skip("no support for single particle")


class TestGadget2GravityCode(_TestGravityCodes):
    length_unit = units.parsec
    speed_unit = units.parsec / units.Myr
    mass_unit = units.MSun

    @property
    def nbody_converter(self):
        return nbody_system.nbody_to_si(1.0 | units.parsec, 100.0 | units.MSun)

    def gravity_code_factory(self):
        return Gadget2

    def test2(self):
        self.skip("no support for setting of radius")

    def test8(self):
        self.skip("no support for collision detection")

    def test9(self):
        self.skip("no support for out of box detection")

    def test10(self):
        self.skip("no support for single particle")



from amuse.test import amusetest
from io import BytesIO
from collections import namedtuple

import os.path
import math
from amuse import io
from amuse.io import gadget
from amuse.io import nemobin
from amuse.units import nbody_system
from amuse.datamodel import Particles


class GadgetFileFormatProcessorTests(amusetest.TestCase):
    header_parts = (
        b'\x00\x01\x00\x00 N\x00\x00 \xa1\x07\x00 N\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\xf6`Q\xc6#\xcc\x9c>\x8d',
        b'\xed\xb5\xa0\xf7\xc6\x90>\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 N\x00\x00 ',
        b'\xa1\x07\x00 N\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
        b'\x00\x00\x00\x00\x00\x00\x01\x00\x00',
    )

    def test1(self):
        header = b''.join(self.header_parts)
        x = gadget.GadgetFileFormatProcessor()
        file = BytesIO(header)
        x.load_header(file)
        print(x.header_struct)
        self.assertEqual(x.header_struct.Npart[0], 20000)
        self.assertEqual(x.header_struct.Npart[1], 500000)
        self.assertEqual(x.header_struct.Npart[2], 20000)
        self.assertEqual(x.header_struct.Npart[3], 0)
        self.assertEqual(x.header_struct.Npart[4], 0)
        self.assertEqual(x.header_struct.Npart[5], 0)

        self.assertEqual(x.header_struct.Massarr[0], 0.0)
        self.assertAlmostRelativeEqual(x.header_struct.Massarr[1], 4.2911501e-07, 8)
        print(x.header_struct.Massarr[2])
        self.assertAlmostRelativeEqual(x.header_struct.Massarr[2], 2.5000000e-07, 8)

        self.assertEqual(x.header_struct.FlagSfr, 0)
        self.assertEqual(x.header_struct.FlagFeedback, 0)
        self.assertEqual(x.header_struct.FlagAge, 0)
        self.assertEqual(x.header_struct.HubbleParam, 0)

    def test2(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gadget_snapshot')
        x = gadget.GadgetFileFormatProcessor()
        file = open(filename, 'rb')
        result = x.load_file(file)
        file.close()
        self.assertEqual(len(result[0]), 1000)
        self.assertEqual(len(result[1]), 10000)

    def test3(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gadget_snapshot')
        x = gadget.GadgetFileFormatProcessor()
        result = io.read_set_from_file(filename, format='gadget')
        self.assertEqual(len(result[0]), 1000)
        self.assertEqual(len(result[1]), 10000)

    def test4(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gassphere_littleendian.dat')
        x = gadget.GadgetFileFormatProcessor()
        result = io.read_set_from_file(filename, format='gadget')
        self.assertEqual(len(result[0]), 1472)
        self.assertEqual(len(result[1]), 0)

    def test5(self):
        options = io.get_options_for_format('gadget')
        found_has_acceleration = False
        for name, description, defaultval in options:
            if name == 'has_acceleration':
                found_has_acceleration = True

        self.assertTrue(found_has_acceleration)

    def test6(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gassphere_littleendian.dat')
        x = gadget.GadgetFileFormatProcessor()
        gas, halo, disk, bulge, stars, bndry = io.read_set_from_file(filename, format='gadget')
        self.assertEqual(len(gas), 1472)
        self.assertEqual(len(halo), 0)
        self.assertEqual(gas[0].key, 1)
        self.assertEqual(gas[1].key, 2)
        self.assertEqual(gas[2].key, 3)
        self.assertEqual(gas[1471].key, 1472)

    def test7(self):
        """test returned ids from gadget file
        for ticket #245.
        All the 'uneven' particles have key "1", and identical velocities/positions. This is incorrect
        upon further inspection, the test file is incorrect
        """
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'ticket245.dat')
        gas, halo, disk, bulge, stars, bndry = io.read_set_from_file(filename, format='gadget')

        self.assertEqual(len(gas), 0)
        self.assertEqual(len(halo), 1324)
        self.assertEqual(len(disk), 0)
        self.assertEqual(len(bulge), 0)
        self.assertEqual(len(stars), 0)
        self.assertEqual(len(bndry), 0)
        self.assertEqual(halo[0].key, 544418538)
        self.assertEqual(halo[1].key, 544511335)
        self.assertEqual(halo[2].key, 544511457)
        self.assertAlmostRelativeEquals(halo[0].velocity[0], -24.785614 | nbody_system.speed, 7)
        print(halo[1].velocity)
        self.assertAlmostRelativeEquals(halo[1].velocity[0], -25.346375 | nbody_system.speed, 7)
        self.assertAlmostRelativeEquals(halo[2].velocity[0], -25.394440 | nbody_system.speed, 7)

    def test8(self):
        """test returned ids from gadget file
        for ticket #245.
        added option to not use the ids as a key, should fix the problem
        for incorrect id's
        """
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'ticket245.dat')
        gas, halo, disk, bulge, stars, bndry = io.read_set_from_file(filename, format='gadget', ids_are_keys=False)

        self.assertEqual(len(gas), 0)
        self.assertEqual(len(halo), 1324)
        self.assertEqual(len(disk), 0)
        self.assertEqual(len(bulge), 0)
        self.assertEqual(len(stars), 0)
        self.assertEqual(len(bndry), 0)
        self.assertEqual(halo[0].id, 544418538)
        self.assertEqual(halo[1].id, 544511335)
        self.assertEqual(halo[2].id, 544511457)
        self.assertAlmostRelativeEquals(halo[0].velocity[0], -24.785614 | nbody_system.speed, 7)
        self.assertAlmostRelativeEquals(halo[1].velocity[0], -25.346375 | nbody_system.speed, 7)
        self.assertAlmostRelativeEquals(halo[2].velocity[0], -25.394440 | nbody_system.speed, 7)

    def test9(self):
        class FakeList(object):
            def __init__(self, _len):
                self._len = _len

            def __len__(self):
                return self._len

        set = (FakeList(20000), FakeList(500000), FakeList(20000), (), (), ())
        x = gadget.GadgetFileFormatProcessor(set=set)
        x.equal_mass_array = (0.0, 4.291150104743886e-07, 2.5e-07, 0.0, 0.0, 0.0) | nbody_system.mass
        file = BytesIO()
        x.store_header(file)
        print(x.header_struct)
        self.assertEqual(x.header_struct.Npart[0], 20000)
        self.assertEqual(x.header_struct.Npart[1], 500000)
        self.assertEqual(x.header_struct.Npart[2], 20000)
        self.assertEqual(x.header_struct.Npart[3], 0)
        self.assertEqual(x.header_struct.Npart[4], 0)
        self.assertEqual(x.header_struct.Npart[5], 0)

        print(repr(file.getvalue()))
        print(repr(b''.join(self.header_parts)))
        self.assertEqual(repr(file.getvalue()[0:30]), repr(b''.join(self.header_parts)[0:30]))

    def test10(self):
        p = Particles(2)
        p[0].position = [1.0, 2.0, 3.0] | nbody_system.length
        p[1].position = [4.0, 5.0, 6.0] | nbody_system.length
        p[0].velocity = [7.0, 8.0, 10.0] | nbody_system.length / nbody_system.time
        p[1].velocity = [11.0, 12.0, 13.0] | nbody_system.length / nbody_system.time
        p.u = [3, 4] | nbody_system.potential
        p.rho = [5, 6] | nbody_system.density
        p.mass = [5, 6] | nbody_system.mass
        x = gadget.GadgetFileFormatProcessor(set=p)
        file = BytesIO()
        x.store_body(file)
        input = BytesIO(file.getvalue())
        positions = x.read_fortran_block_float_vectors(input)
        self.assertEqual(positions[0], [1.0, 2.0, 3.0])
        self.assertEqual(positions[1], [4.0, 5.0, 6.0])
        velocities = x.read_fortran_block_float_vectors(input)
        self.assertEqual(velocities[0], [7.0, 8.0, 10.0])
        self.assertEqual(velocities[1], [11.0, 12.0, 13.0])
        ids = x.read_fortran_block_ulongs(input)
        self.assertEqual(ids[0], p[0].key)
        self.assertEqual(ids[1], p[1].key)
        masses = x.read_fortran_block_floats(input)
        self.assertEqual(masses[0], 5)
        self.assertEqual(masses[1], 6)
        u = x.read_fortran_block_floats(input)
        self.assertEqual(u[0], 3)
        self.assertEqual(u[1], 4)

    def test11(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gassphere_littleendian.dat')
        gas, halo, disk, bulge, stars, bndry = io.read_set_from_file(filename, format='gadget')
        self.assertEqual(len(gas), 1472)
        self.assertEqual(len(halo), 0)
        self.assertEqual(gas[0].key, 1)
        self.assertEqual(gas[1].key, 2)
        self.assertEqual(gas[2].key, 3)
        self.assertEqual(gas[1471].key, 1472)
        self.assertAlmostRelativeEquals(gas[0:5].x, [-0.0713372901082, 0.0713372901082, -0.21178227663, -0.0698266476393, 0.0698266476393] | nbody_system.length, 7)
        self.assertAlmostRelativeEquals(gas[0:5].u, [0.0500000007451, 0.0500000007451, 0.0500000007451, 0.0500000007451, 0.0500000007451] | (nbody_system.length / nbody_system.time)**2, 7)

        outputfilename = 'gadgettest.output'
        try:
            io.write_set_to_file((gas, halo, disk, bulge, stars, bndry), outputfilename, format='gadget', ids_are_long=False)

            gas, halo, disk, bulge, stars, bndry = io.read_set_from_file(outputfilename, format='gadget')
            self.assertEqual(len(gas), 1472)
            self.assertEqual(len(halo), 0)
            self.assertEqual(gas[0].key, 1)
            self.assertEqual(gas[1].key, 2)
            self.assertEqual(gas[2].key, 3)
            self.assertEqual(gas[1471].key, 1472)
        finally:
            if os.path.exists(outputfilename):
                os.remove(outputfilename)

    def test12(self):
        print("Test return_header for Gadget read_set_from_file")
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'gassphere_littleendian.dat')
        data = io.read_set_from_file(filename, format='gadget', return_header=False)  # (default)
        self.assertTrue(isinstance(data, tuple))
        self.assertEqual(data.__doc__, "GadgetData(gas, halo, disk, bulge, stars, bndry)")

        data = io.read_set_from_file(filename, format='gadget', return_header=True)
        self.assertTrue(isinstance(data, tuple))
        self.assertEqual(data.__doc__, "GadgetData(gas, halo, disk, bulge, stars, bndry, "
            "Npart, Massarr, Time, Redshift, FlagSfr, FlagFeedback, Nall, FlagCooling, "
            "NumFiles, BoxSize, Omega0, OmegaLambda, HubbleParam, FlagAge, FlagMetals, "
            "NallHW, flag_entr_ics)")

        self.assertEqual(len(data.gas), 1472)
        self.assertEqual(len(data.halo), 0)
        self.assertEqual(data.gas[0].key, 1)
        self.assertEqual(data.gas[1].key, 2)
        self.assertEqual(data.gas[2].key, 3)
        self.assertEqual(data.gas[1471].key, 1472)
        self.assertAlmostRelativeEquals(data.gas[0:5].x, [-0.0713372901082, 0.0713372901082, -0.21178227663, -0.0698266476393, 0.0698266476393] | nbody_system.length, 7)
        self.assertAlmostRelativeEquals(data.gas[0:5].u, [0.0500000007451, 0.0500000007451, 0.0500000007451, 0.0500000007451, 0.0500000007451] | (nbody_system.length / nbody_system.time)**2, 7)

        self.assertEqual(data.Npart, (1472, 0, 0, 0, 0, 0))
        self.assertEqual(data.Time, 0.0)
        self.assertEqual(data.Redshift, 0.0)

    def test13(self):
        print("Test convert_gadget_w_to_velocity and return_header for Gadget read_set_from_file")
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'tiny_lcdm_data_littleendian.dat')
        data = io.read_set_from_file(filename, format='gadget', return_header=False, convert_gadget_w_to_velocity=False)  # (default)
        self.assertTrue(isinstance(data, tuple))
        self.assertEqual(data.__doc__, "GadgetData(gas, halo, disk, bulge, stars, bndry)")
        self.assertEqual(len(data.gas), 32)
        self.assertEqual(len(data.halo), 32)
        self.assertEqual(data.gas[0].key, 1)
        self.assertEqual(data.halo[0].key, 32**3 + 1)
        self.assertAlmostRelativeEquals(data.gas[:3].position, [[395.23443604, 395.75210571, 1244.31152344],
            [310.17266846, 440.21728516, 2817.06396484], [191.95669556, 465.57223511, 4430.20068359]] | nbody_system.length, 7)

        data_converted = io.read_set_from_file(filename, format='gadget', return_header=True, convert_gadget_w_to_velocity=True)
        self.assertTrue(isinstance(data_converted, tuple))
        self.assertEqual(data_converted.__doc__, "GadgetData(gas, halo, disk, bulge, stars, bndry, "
            "Npart, Massarr, Time, Redshift, FlagSfr, FlagFeedback, Nall, FlagCooling, "
            "NumFiles, BoxSize, Omega0, OmegaLambda, HubbleParam, FlagAge, FlagMetals, "
            "NallHW, flag_entr_ics)")

        self.assertEqual(len(data_converted.gas), 32)
        self.assertEqual(len(data_converted.halo), 32)
        self.assertEqual(data_converted.gas[0].key, 1)
        self.assertEqual(data_converted.halo[0].key, 32**3 + 1)
        self.assertEqual(data_converted.Npart, (32, 32, 0, 0, 0, 0))
        self.assertEqual(data_converted.Time, 1/11.0)
        self.assertEqual(data_converted.Redshift, 10.0)
        self.assertEqual(data.gas.position, data_converted.gas.position)
        self.assertAlmostRelativeEquals(data.gas.velocity, math.sqrt(data_converted.Time) * data_converted.gas.velocity, 7)


class NemoBinaryFileFormatProcessorTests(amusetest.TestCase):

    def test1(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        nemofile = nemobin.NemoBinaryFile(file)
        tagcharacter, tagstring, dim, mustswap = nemofile.get_item_header()
        self.assertEqual(tagcharacter, 'c')
        self.assertEqual(tagstring, 'Headline')
        self.assertEqual(len(dim), 1)
        self.assertEqual(dim[0], 28)
        file.close()

    def test2(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        nemofile = nemobin.NemoBinaryFile(file)
        item = nemofile.read_item()

        self.assertEqual(item.data, "init_xrandom: seed used 123")
        file.close()

    def test3(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        nemofile = nemobin.NemoBinaryFile(file)
        data = nemofile.read()
        file.close()
        self.assertEqual(len(data), 3)
        tags = list(data.keys())
        self.assertEqual(tags[0], 'Headline')
        self.assertEqual(tags[1], 'History')
        self.assertEqual(tags[2], 'SnapShot')
        self.assertEqual(data['History'][0].data, 'mkplummer out=plummer128.nemo nbody=128 seed=123 VERSION=2.8b')

        self.assertEqual(len(data['SnapShot'][0].data), 2)
        tags = list(data['SnapShot'][0].data.keys())
        self.assertEqual(tags[0], 'Parameters')
        self.assertEqual(tags[1], 'Particles')

    def test4(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        x = nemobin.NemoBinaryFileFormatProcessor()

        set = x.load_file(file).previous_state()
        file.close()
        self.assertEqual(len(set), 128)
        self.assertEqual(set.get_timestamp(), 0.0 | nbody_system.time)
        self.assertAlmostRelativeEquals(set.kinetic_energy(), 0.230214395174 | nbody_system.energy, 8)
        self.assertAlmostRelativeEquals(set.potential_energy(G=nbody_system.G), -0.473503040144 | nbody_system.energy, 8)

    def test5(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        nemofile = nemobin.NemoBinaryFile(file)
        data = nemofile.read()
        file.close()

        outputfile = BytesIO()
        nemooutputfile = nemobin.NemoBinaryFile(outputfile)
        nemooutputfile.write(data)
        string = outputfile.getvalue()
        outputfile.close()
        inputfile = BytesIO(string)
        nemoinputfile = nemobin.NemoBinaryFile(inputfile)
        tagcharacter, tagstring, dim, mustswap = nemoinputfile.get_item_header()
        self.assertEqual(tagcharacter, 'c')
        self.assertEqual(tagstring, 'Headline')
        self.assertEqual(len(dim), 1)
        self.assertEqual(dim[0], 28)
        inputfile.close()

    def test6(self):
        inputfile = BytesIO()
        nemoinputfile = nemobin.NemoBinaryFile(inputfile)
        data = nemoinputfile.read()
        self.assertEqual(len(data), 0)

    def test7(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        nemofile = nemobin.NemoBinaryFile(file)
        data = nemofile.read()
        file.close()

        outputfile = BytesIO()
        nemooutputfile = nemobin.NemoBinaryFile(outputfile)
        nemooutputfile.write(data)
        string = outputfile.getvalue()
        outputfile.close()

        file = open(filename, 'rb')
        original = file.read()
        file.close()

        self.assertEqual(len(original), len(string))
        self.assertEqual(original, string)

    def test8(self):
        directory_name = os.path.dirname(__file__)
        filename = os.path.join(directory_name, 'plummer128.nemo')
        file = open(filename, 'rb')
        x = nemobin.NemoBinaryFileFormatProcessor()

        set = x.load_file(file)
        file.close()

        outputfile = BytesIO()

        y = nemobin.NemoBinaryFileFormatProcessor()
        y.set = set
        y.store_file(outputfile)
        string = outputfile.getvalue()
        outputfile.close()

        inputfile = BytesIO(string)
        x = nemobin.NemoBinaryFileFormatProcessor()
        set = x.load_file(inputfile)
        inputfile.close()
        self.assertEqual(len(set), 128)
        self.assertAlmostRelativeEquals(set.kinetic_energy(), 0.230214395174 | nbody_system.energy, 8)
        self.assertAlmostRelativeEquals(set.potential_energy(G=nbody_system.G), -0.473503040144 | nbody_system.energy, 8)

    def test9(self):
        filename = os.path.join(os.path.dirname(__file__), 'plummer128.nemo')
        particles = io.read_set_from_file(filename, format="nemobin")
        self.assertEqual(len(particles), 128)
        self.assertAlmostEqual(particles.total_mass(), 1.0 | nbody_system.mass)
        self.assertAlmostEqual(particles.center_of_mass(), 0.0 | nbody_system.length)
        self.assertAlmostEqual(particles.center_of_mass_velocity(), 0.0 | nbody_system.speed)
        self.assertAlmostEqual(particles.kinetic_energy(), 0.230214395174 | nbody_system.energy)



import os.path
import numpy
import subprocess
from amuse.community import *
from amuse.test.amusetest import TestWithMPI
from amuse.community.halogen.interface import HalogenInterface, Halogen
import amuse.community.halogen

# Change the default for some Halogen(-Interface) keyword arguments:
default_options = dict()
# default_options = dict(redirection = "none")


class HalogenInterfaceTests(TestWithMPI):

    def test1(self):
        print("Testing HalogenInterface initialization")
        instance = HalogenInterface(**default_options)
        self.assertEqual(instance.initialize_code(), 0)
        self.assertEqual(instance.set_model_alpha(2.0), 0)
        self.assertEqual(instance.set_model_beta(5.0), 0)
        self.assertEqual(instance.set_model_gamma(0.0), 0)
        self.assertEqual(instance.set_target_number_of_particles(1000), 0)
        self.assertEqual(instance.commit_parameters(), 0)
        self.assertEqual(instance.cleanup_code(), 0)
        instance.stop()

    def test2(self):
        print("Testing HalogenInterface parameters")
        instance = HalogenInterface(**default_options)
        self.assertEqual(instance.initialize_code(), 0)

        self.assertEqual(instance.set_model_alpha(2.0), 0)
        self.assertEqual(instance.set_model_beta(5.0),  0)
        self.assertEqual(instance.set_model_gamma(0.0), 0)
        self.assertEqual(instance.set_target_number_of_particles(100), 0)
        self.assertEqual(instance.set_random_seed(1), 0)
        self.assertEqual(instance.set_total_mass(9.0),  0)
        self.assertEqual(instance.set_scale_radius(9.0), 0)
        self.assertEqual(instance.set_cutoff_radius(9.0), 0)
        self.assertEqual(instance.set_black_hole_mass(9.0), 0)
        self.assertEqual(instance.set_do_exact_virial_radius_flag(1.0), 0)

        self.assertEqual([2.0, 0], list(instance.get_model_alpha().values()))
        self.assertEqual([5.0, 0], list(instance.get_model_beta().values()))
        self.assertEqual([0.0, 0], list(instance.get_model_gamma().values()))
        self.assertEqual([100, 0], list(instance.get_target_number_of_particles().values()))
        self.assertEqual([1, 0], list(instance.get_random_seed().values()))
        self.assertEqual([9.0, 0], list(instance.get_total_mass().values()))
        self.assertEqual([9.0, 0], list(instance.get_scale_radius().values()))
        self.assertEqual([9.0, 0], list(instance.get_cutoff_radius().values()))
        self.assertEqual([9.0, 0], list(instance.get_black_hole_mass().values()))
        self.assertEqual([1.0, 0], list(instance.get_do_exact_virial_radius_flag().values()))

        self.assertEqual(instance.cleanup_code(), 0)
        instance.stop()

    def test3(self):
        print("Testing HalogenInterface generate_particles")
        number_of_particles = 100000
        instance = HalogenInterface(**default_options)
        self.assertEqual(instance.initialize_code(), 0)
        self.assertEqual(instance.set_model_alpha(2.0), 0)
        self.assertEqual(instance.set_model_beta(5.0), 0)
        self.assertEqual(instance.set_model_gamma(0.0), 0)
        self.assertEqual(instance.set_target_number_of_particles(number_of_particles), 0)
        self.assertEqual(instance.set_random_seed(1), 0)
        self.assertEqual(instance.set_output_path(instance.get_output_directory()), 0)
        self.assertEqual(instance.commit_parameters(), 0)

        self.assertEqual(list(instance.get_number_of_particles_updated().values()), [0, 0])
        self.assertEqual(instance.generate_particles(), 0)
        self.assertEqual(list(instance.get_number_of_particles_updated().values()), [number_of_particles, 0])

        masses, errors = instance.get_mass(range(number_of_particles))
        self.assertEqual(errors, numpy.zeros(number_of_particles))
        self.assertEqual(masses, numpy.ones(number_of_particles)/number_of_particles)

        x_positions, y_positions, z_positions, errors = instance.get_position(range(number_of_particles))
        self.assertEqual(errors, numpy.zeros(number_of_particles))
        self.assertAlmostEqual(numpy.array([numpy.mean(x_positions), numpy.mean(y_positions),
            numpy.mean(z_positions)]), numpy.array([0.0]*3))
        self.assertAlmostEqual(numpy.array([numpy.mean(abs(x_positions)), numpy.mean(abs(y_positions)),
            numpy.mean(abs(z_positions))]), numpy.array([1.0]*3), 1)

        x_velocities, y_velocities, z_velocities, errors = instance.get_velocity(range(number_of_particles))
        self.assertEqual(errors, numpy.zeros(number_of_particles))
        self.assertAlmostEqual(numpy.array([numpy.mean(x_velocities), numpy.mean(y_velocities),
            numpy.mean(z_velocities)]), numpy.array([0.0]*3))
        self.assertAlmostEqual(numpy.array([numpy.mean(abs(x_velocities)), numpy.mean(abs(y_velocities)),
            numpy.mean(abs(z_velocities))]), numpy.array([0.25]*3), 1)

        self.assertEqual(instance.cleanup_code(), 0)
        instance.stop()

    def test4(self):
        print("Testing HalogenInterface output file name and directory")
        instance = HalogenInterface(**default_options)
        self.assertEqual(instance.initialize_code(), 0)

        self.assertEqual(instance.set_model_alpha(2.0), 0)
        self.assertEqual(instance.set_model_beta(5.0), 0)
        self.assertEqual(instance.set_model_gamma(0.0), 0)
        self.assertEqual(instance.set_target_number_of_particles(1000), 0)
        self.assertEqual(instance.set_random_seed(1), 0)
        self.assertEqual(instance.set_write_output_flag(1.0), 0)

        self.assertEqual(list(instance.get_output_basename().values()), ["halogen", 0])
        self.assertEqual(list(instance.get_output_path().values()), ["./", 0])

        self.assertEqual(instance.set_output_basename("oops_this_output_basename_is_way_too_long"*2), -1)
        self.assertEqual(list(instance.get_output_basename().values()), ["halogen", 0])
        self.assertEqual(instance.set_output_path("/oops/this/output/path/has/way/too/many/subdirs"*6), -1)
        self.assertEqual(list(instance.get_output_path().values()), ["./", 0])

        self.assertEqual(instance.set_output_basename("test"), 0)
        self.assertEqual(instance.set_output_path(instance.get_output_directory()), 0)

        self.assertEqual(list(instance.get_output_basename().values()), ["test", 0])
        self.assertEqual(list(instance.get_output_path().values()),
            [os.path.join(instance.get_output_directory(), ""), 0])

        self.assertEqual(instance.commit_parameters(), 0)
        outputfile = os.path.join(instance.get_output_directory(), "test.IC.ascii")
        if os.path.exists(outputfile):
            os.remove(outputfile)
        self.assertEqual(instance.generate_particles(), 0)
        self.assertTrue(os.path.exists(outputfile))

        halogen4muse_path = os.path.join(os.path.dirname(amuse.community.halogen.__file__), 'src', 'halogen4muse')

        if not os.path.exists(halogen4muse_path) or not os.access(halogen4muse_path, os.X_OK):
            return

        process = subprocess.Popen([
            halogen4muse_path,
            '-a', '2',
            '-b', '5',
            '-c', '0',
            '-N', '1000',
            '-name', 'test_stand_alone',
            '-randomseed', '1'
            ], cwd=instance.get_output_directory(), stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        process.communicate()

        self.compare_files(
            os.path.join(instance.get_output_directory(), "test.IC.ascii"),
            os.path.join(instance.get_output_directory(), "test_stand_alone.IC.ascii")
        )
        stdoutput = subprocess.Popen(["diff", "test.out", "test_stand_alone.out"],
            cwd=instance.get_output_directory(), stdout=subprocess.PIPE).communicate()[0]
        stdoutput = stdoutput.decode()
        self.assertTrue("< N/A (executed by AMUSE)" in stdoutput)
        self.assertTrue("halogen4muse -a 2 -b 5 -c 0 -N 1000 -name test_stand_alone -randomseed 1" in stdoutput)

        self.assertEqual(instance.cleanup_code(), 0)
        instance.stop()

    def compare_files(self, filename1, filename2):
        with open(filename1) as stream1:
            lines1 = stream1.readlines()
        with open(filename2) as stream2:
            lines2 = stream2.readlines()

        self.assertTrue(len(lines1), len(lines2))
        for line1, line2 in zip(lines1, lines2):
            columns1 = line1.split()
            columns2 = line2.split()
            self.assertTrue(len(columns1), len(columns2))
            if (len(columns1)) == 0:
                continue
            self.assertEqual(int(columns1[0]), int(columns2[0]))
            for column1, column2 in zip(columns1[1:], columns2[1:]):
                self.assertAlmostRelativeEquals(
                    float(column1),
                    float(column2),
                    8
                )


class HalogenTests(TestWithMPI):

    default_unit_converter = nbody_system.nbody_to_si(1.0 | units.kpc, 1.0e6 | units.MSun)

    def test1(self):
        print("Testing Halogen initialization")
        instance = Halogen(**default_options)
        instance.initialize_code()
        instance.cleanup_code()
        instance.stop()

    def test2(self):
        print("Testing Halogen parameters (with unit converter)")
        instance = Halogen(self.default_unit_converter, **default_options)
        instance.initialize_code()

        for par, value in [('do_exact_virial_radius_flag', False),
                ('outputgridr_flag', False), ('outputgriddf_flag', False),
                ('write_output_flag', False)]:
            self.assertTrue(value is getattr(instance.parameters, par))
            setattr(instance.parameters, par, not value)
            self.assertFalse(value is getattr(instance.parameters, par))

        for par, value in [('alpha', -1.0), ('beta', -1.0), ('gamma', -1.0),
                ('number_of_particles', -1), ('random_seed', 42)]:
            self.assertEqual(value, getattr(instance.parameters, par))
            setattr(instance.parameters, par, 1)
            self.assertEqual(1, getattr(instance.parameters, par))

        for par, value in [('total_mass', 1.0 | nbody_system.mass),
                ('scale_radius',    1.0 | nbody_system.length),
                ('cutoff_radius',  -1.0 | nbody_system.length),
                ('black_hole_mass', 0.0 | nbody_system.mass)]:
            self.assertEqual(instance.unit_converter.to_si(value),
                getattr(instance.parameters, par))
            setattr(instance.parameters, par, 3.0 | value.unit)
            self.assertEqual(instance.unit_converter.to_si(3.0 | value.unit),
                getattr(instance.parameters, par))

        for par, value in [('output_directory', os.path.join(instance.get_output_directory(), "")),
                ('output_basename', "halogen")]:
            self.assertEqual(value, getattr(instance.parameters, par))
            setattr(instance.parameters, par, 'test/')
            self.assertEqual("test/", getattr(instance.parameters, par))

        instance.cleanup_code()
        instance.stop()

    def test3(self):
        print("Testing Halogen parameters (nbody units, no converter)")
        instance = Halogen(**default_options)
        instance.initialize_code()

        for par, value in [('total_mass', 1.0 | nbody_system.mass),
                ('scale_radius',    1.0 | nbody_system.length),
                ('cutoff_radius',  -1.0 | nbody_system.length),
                ('black_hole_mass', 0.0 | nbody_system.mass)]:
            self.assertEqual(value, getattr(instance.parameters, par))
            setattr(instance.parameters, par, 3.0 | value.unit)
            self.assertEqual(3.0 | value.unit, getattr(instance.parameters, par))
        instance.cleanup_code()
        instance.stop()

    def test4(self):
        print("Testing Halogen generate_particles")
        number_of_particles = 100
        instance = Halogen(**default_options)
        instance.initialize_code()
        instance.parameters.alpha = 2.0
        instance.parameters.beta = 5.0
        instance.parameters.gamma = 0.0
        instance.parameters.number_of_particles = number_of_particles
        instance.parameters.random_seed = 1
        instance.commit_parameters()
        instance.generate_particles()
        self.assertEqual(len(instance.particles), number_of_particles)
        self.assertAlmostEqual(instance.particles.total_mass(), 1.0 | nbody_system.mass)
        self.assertAlmostEqual(instance.particles.kinetic_energy(),
            0.17345836639 | nbody_system.energy)
        self.assertAlmostEqual(instance.particles.potential_energy(G=nbody_system.G),
            -0.310395778644 | nbody_system.energy)
        self.assertAlmostEqual(instance.particles.virial_radius(),
            1.61084664935 | nbody_system.length)

        instance.cleanup_code()
        instance.stop()

    def test5(self):
        print("Testing Halogen generate_particles: generate multiple sets")
        number_of_particles = 1000
        instance = Halogen(**default_options)
        instance.initialize_code()
        instance.parameters.alpha = 2.0
        instance.parameters.beta = 5.0
        instance.parameters.gamma = 0.0
        instance.parameters.number_of_particles = number_of_particles
        instance.parameters.random_seed = 1
        instance.commit_parameters()

        instance.generate_particles()
        set1 = instance.particles.copy()
        self.assertEqual(len(set1), number_of_particles)

        instance.parameters.random_seed = 1
        instance.generate_particles()
        set2 = instance.particles.copy()
        self.assertEqual(len(set2), number_of_particles)
        # Since a (any would do!) parameter was changed, recommit_parameters was
        # called, re-seeding, and the result should be the same:
        for attribute in ["mass", "x", "y", "z", "vx", "vy", "vz"]:
            self.assertEqual(getattr(set1, attribute), getattr(set2, attribute))

        instance.generate_particles()
        # No parameter change: draw the next random set of particles
        set3 = instance.particles.copy()
        self.assertEqual(len(set3), number_of_particles)
        self.assertEqual(set1.mass, set3.mass)
        self.assertRaises(self.failureException, self.assertEqual, set1.x, set3.x)
        self.assertIsOfOrder(abs(set1.x).median(), abs(set3.x).median(), 1)
        self.assertAlmostEqual(abs(set1.vy).median(), abs(set3.vy).median(), 1)

        instance.cleanup_code()
        instance.stop()

    def test6(self):
        print("Testing Halogen state")
        number_of_particles = 1000

        print("First do everything manually:")
        instance = Halogen(**default_options)
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.initialize_code()
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        instance.parameters.alpha = 2.0
        instance.parameters.beta = 5.0
        instance.parameters.gamma = 0.0
        instance.parameters.number_of_particles = number_of_particles
        instance.parameters.random_seed = 1
        instance.commit_parameters()
        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')
        instance.overridden().generate_particles()
        self.assertEqual(instance.get_name_of_current_state(), 'UPDATE')
        instance.invoke_state_change_updated()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        self.assertEqual(len(instance.particles), number_of_particles)
        instance.cleanup_code()
        self.assertEqual(instance.get_name_of_current_state(), 'END')
        instance.stop()

        print("initialize_code(), (re)commit_parameters(), update_particle_set(), "
            "and cleanup_code() should be called automatically:")
        instance = Halogen(**default_options)
        self.assertEqual(instance.get_name_of_current_state(), 'UNINITIALIZED')
        instance.parameters.alpha = 2.0
        instance.parameters.beta = 5.0
        instance.parameters.gamma = 0.0
        instance.parameters.number_of_particles = number_of_particles
        instance.parameters.random_seed = 1
        self.assertEqual(instance.get_name_of_current_state(), 'INITIALIZED')
        self.assertEqual(instance.get_number_of_particles_updated(), 0)
        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')
        instance.parameters.random_seed = 2
        self.assertEqual(instance.get_name_of_current_state(), 'CHANGE_PARAMETERS_EDIT')
        self.assertEqual(instance.get_number_of_particles_updated(), 0)
        self.assertEqual(instance.get_name_of_current_state(), 'EDIT')
        instance.generate_particles()
        self.assertEqual(instance.get_name_of_current_state(), 'RUN')
        self.assertEqual(len(instance.particles), number_of_particles)
        self.assertEqual(instance.get_number_of_particles_updated(), 0)
        instance.stop()
        self.assertEqual(instance.get_name_of_current_state(), 'STOPPED')

    def test7(self):
        print("Testing Halogen error handling")
        number_of_particles = 1000
        instance = Halogen(**default_options)
        instance.initialize_code()
        self.assertRaises(exceptions.AmuseException, instance.commit_parameters, expected_message="Error when calling 'commit_parameters' of a 'Halogen', errorcode is -2, error is "
            "'Missing or bad parameter for halo (see amuse/community/halogen/src/doc for details on required parameters).'")
        instance.parameters.alpha = 2.0
        instance.parameters.beta = 5.0
        instance.parameters.gamma = 5.0
        instance.parameters.number_of_particles = number_of_particles
        instance.parameters.random_seed = 1
        self.assertRaises(exceptions.AmuseException, instance.commit_parameters, expected_message="Error when calling 'commit_parameters' of a 'Halogen', errorcode is -2, error is "
            "'Missing or bad parameter for halo (see amuse/community/halogen/src/doc for details on required parameters).'")
        instance.parameters.gamma = -0.5
        self.assertRaises(exceptions.AmuseException, instance.commit_parameters, expected_message="Error when calling 'commit_parameters' of a 'Halogen', errorcode is -2, error is "
            "'Missing or bad parameter for halo (see amuse/community/halogen/src/doc for details on required parameters).'")
        instance.parameters.gamma = 0.0
        instance.parameters.beta = 2.0
        self.assertRaises(exceptions.AmuseException, instance.commit_parameters, expected_message="Error when calling 'commit_parameters' of a 'Halogen', errorcode is -2, error is "
            "'Missing or bad parameter for halo (see amuse/community/halogen/src/doc for details on required parameters).'")
        instance.parameters.beta = 5.0
        instance.commit_parameters()

        instance.cleanup_code()
        instance.stop()
